﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using WindowsFormsApplication1.MyMath;
using WindowsFormsApplication1.SNetwork;

namespace WindowsFormsApplication1.Events
{
    public class Event : WithActers
    {
        public static readonly int MEET = 1;
        public static readonly int SEPARATION = -1;
        public static readonly int UNDEFINED = 0;
        public static readonly int ENDSMEETING = 3;
        public static readonly int OVERLAP = 2;
        public double angle;
        public int type;
        public MyPoint location;

        public List<PairOfActers> ActersPairs = new List<PairOfActers>(10);

        public Event(double angle, int type, MyPoint location)
        {
            this.angle = angle % 360;
            this.type = type;
            this.location = location;
        }

        public override int GetHashCode()
        {
            return base.GetHashCode();
        }

        /// <summary>
        /// Ak sa rovnaju uhly a miesta udalosti, potom su povazovane za tu istu udalost
        /// </summary>
        /// <param name="obj"></param>
        /// <returns></returns>
        public override bool Equals(Object obj)
        {
            if (!(obj is Event)) return false;
            Event p = (Event)obj;
            if ( Math.Abs(p.angle - this.angle)<Parameters.angleDeviation && p.location.Equals(this.location) && p.type == this.type)
                return true;
            return false;
        }

        public void addActersPair(PairOfActers s)
        {
            if (!ActersPairs.Contains(s))
                ActersPairs.Add(s);
        }

        public override string ToString()
        {
            String s = "EVENT: ";
                if (this.type == Event.MEET)
                    s= s +"stretnutie ;";
                if (this.type == Event.SEPARATION)
                    s= s +"rozidenie ;";
                if (this.type == Event.UNDEFINED)
                    s= s +"nedefinovane ;";
                if (this.type == Event.OVERLAP)
                    s = s + "prekrytie ;";
                s = s +"uhol: "+ angle.ToString();
            return s+ " " + base.ToString() ;
        }


        /// <summary>
        /// Zalezi na poradi... hladame rozidenia s vacsim uhlom ako angle1 a mensim ako angle2
        /// </summary>
        /// <param name="angle1">prvy uhol</param>
        /// <param name="angle2">druhy nasledovny uhol</param>
        /// <param name="Events">Rozidenia medzi tymito uhlami</param>
        /// <returns></returns>
        public static List<Event> getSepartionsBetweenAngles(double angle1, double angle2, List<Event> Events)
        {
            List<Event> result = new List<Event>(10);
            if (Events == null)
            {
                throw new Exception("Nemozno zistovat ekvivalecie SP, ak je zoznam udalosti NULL!");
            }

            foreach (Event ev in Events)
            {
                //nemusime zohladit, ze je mozne sa pytat na udalosti medzi uhlami ako napr 350 a 10
                //pretoze uhol udalosti nemoze byt rovny, vacsi ako 360
                if (angle1 < angle2)
                {
                    if ( ( angle1 <=ev.angle   && angle2> ev.angle  ) && ev.type == Event.SEPARATION)
                    {
                        result.Add(ev);
                    }
                }

                if (angle2 < angle1 )
                {
                    if (( angle1 <= ev.angle  || angle2 < ev.angle  ) && ev.type == Event.SEPARATION)
                    {
                        result.Add(ev);
                    }
                }
                
            }
            return result;
        }


        public static List<Event> getListsTogether(List<Event> List1, List<Event> List2)
        {
            List<Event> list = new List<Event>(List1.Count + List2.Count);

            foreach (Event t in List1)
            {
                if (!list.Contains(t))
                {
                    list.Add(t);
                }
                else
                {
                    list.ElementAt(list.IndexOf(t)).addActers(t.Acters);
                }
            }
            foreach (Event t in List2)
            {
                if (!list.Contains(t))
                {
                    list.Add(t);
                }
                else
                {
                    list.ElementAt(list.IndexOf(t)).addActers(t.Acters);
                }
            }
            return list;
        }
    }

    class EndsMeeting
    {
        public MyPoint mp;
        public double angle;

        public EndsMeeting(MyPoint myPoint, double angle)
        {
            this.mp = myPoint;
            this.angle = angle;
        }
    }
}
