﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using WindowsFormsApplication1.MyMath;
using WindowsFormsApplication1.SNetwork;

namespace WindowsFormsApplication1.Graph
{
        //http://www.vcskicks.com/representing-graphs.php

        public class Graph<T>
        {
            List<List<Vertex<T>>> cycles = new List<List<Vertex<T>>>();
            public string name = "graph";
            bool[][] matrix;
            public List<Vertex<T>> Vertices;
            public List<Vertex<T>> SpecialVertices;
            //ak je to specialny graf sem si zapiseme uhol k akej konfiguracii je vytvorena pridavna, ktora obsahuje tento graf
            public double angleOfConf = 0;
            public List<List<T>> Components = null;

            public Graph<T> copyGraph(){
                Graph<T> result = new Graph<T>();
                foreach (Vertex<T> v in Vertices)
                {
                    result.addVertex(v.Data);
                }

                foreach (Vertex<T> v in Vertices)
                {
                    result.addVertex(v.Data);
                    foreach (Vertex<T> n in v.Edges)
                    {
                        result.addVertex(n.Data);
                        result.addEdge(n.Data, v.Data);
                    }
                }
                return result;
            }


            public void findComponents()
            {
                Components = GraphAlg<T>.GetComponents(this);
            }

            public void setName(String s)
            {
                name = s;
            }

            public double getDistanceOfVertices(int v1Id, int v2Id)
            {
                Vertex<T> v1 = GetByIndex(v1Id);

                Vertex<T> v2 = GetByIndex(v2Id);

                if (v1.Data is MyPoint && v2.Data is MyPoint)
                {
                    return Distances.distance(v1.Data,v2.Data);
                }
                throw new Exception("Nevhodne volanie metody pre vzdialenost vrcholov - da sa len pre typ dat MyPoint");

            }

            public Graph()
            {
                SpecialVertices = new List<Vertex<T>>();
                Vertices = new List<Vertex<T>>(400);
                matrix = new bool[1000][];
                for (int x = 0; x < matrix.Length; x++)
                {
                    bool[] newarr = new bool[1000];
                    for (int y = 0; y < newarr.Length; y++)
                    {
                        newarr[y] = false;
                    }
                    matrix[x] = newarr;
                }
                visited = new List<Vertex<T>>();
            }

            public override String ToString()
            {
                String s;
                s = "_____________________________________ \r\n  Graf pre konfiguráciu s uhlom "+ this.angleOfConf.ToString();
                s = s + "\r\nVrcholy: ";
                foreach (Vertex<T> v in Vertices)
                {
                    s = s + v.Index.ToString() + ": " + v.ToString() + ",";
                }
                s = s + "\r\nHrany: ";
                foreach (Vertex<T> v in Vertices)
                {
                    foreach (Vertex<T> n in this.neighbours(v))
                    {
                        s = s + " [" + v.Index.ToString() + ";" + n.Index.ToString() + "] ";
                    }
                }
              /*  s = s + "\r\nSorted Edges:";
                foreach (Vertex<T> v in Vertices)
                {
                    s = s + "\r\n" + v.Index + ": ";
                    foreach (EdgeWithAngle n in v.EdgesWithAngles)
                    {
                        s = s + " [" + v.Index.ToString() + ";" + n.indexEVertex.ToString() + "] " + " uhol " + Math.Round(n.angle, 3).ToString();
                    }
                }*/
                s = s + "\r\n" + "_____________________________________";
                return s;
            }

            public void addVertexList(List<T> listOfVerexes)
            {
                foreach (T vertex in listOfVerexes)
                {
                    addVertex(vertex);
                }
            }

            public List<Vertex<T>> ListOfIndexesToListOfVertices(List<Vertex<T>> l)
            {
                List<Vertex<T>> result = new List<Vertex<T>>();
                foreach (Vertex<T> i in l)
                {
                    result.Add(i);
                }
                return result;
            }


            public Vertex<T> addSpecialVertex(T t)
            {
                
                Vertex<T> result = addVertex(t);
                
                SpecialVertices.Add(result);
                return result;

            }

            public Vertex<T> addVertex(T t)
            {
                int ind = GetIndexOfVertexWithData(t);
                if (ind == -1)
                {
                    for (int i = 0; i < 1000; i++)
                    {
                        if (GetByIndex(i) == null)
                        {
                            Vertex<T> v = new Vertex<T>();
                            v.Data = t;
                            v.Index = i;
                            Vertices.Add(v);
                            return v;
                        }
                    }
                    throw new System.ArgumentException("Graph is full. It's allowed 1000 elements.");
                }
                else return GetByIndex(ind);
            }


            public void deleteVertex(int id)
            {
                Vertex<T> v = GetByIndex(id);
                foreach (Vertex<T> ver in v.Edges)
                {
                    ver.Edges.Remove(v);
                }

                foreach (Vertex<T> ver in Vertices)
                {
                    ver.EdgesWithAngles.Clear();
                }
                Vertices.Remove(v);

                for (int i = 0; i < 1000; i++)
                {
                    matrix[i][id] = false;
                    matrix[id][i] = false;
                }

            }

            public Vertex<T> GetByIndex(int id)
            {
                foreach (Vertex<T> v in Vertices)
                {
                    if (v.Index == id) return v;
                }
                return null;
            }

            public int GetIndexOfVertexWithData(T t)
            {
                foreach (Vertex<T> v1 in Vertices)
                {
                    if (v1.Data.Equals(t))
                    {
                        return v1.Index;
                    }
                }
                return -1;
            }

            public Vertex<T> GetVertexWithData(T t)
            {
                foreach (Vertex<T> v1 in Vertices)
                {
                    if (v1.Data.Equals(t))
                    {
                        return v1;
                    }
                }
                return null;
            }

            public void addEdge(T t1, T t2)
            {
                Vertex<T> v1 = GetVertexWithData(t1);
                Vertex<T> v2 = GetVertexWithData(t2);
                if (v1.Equals(v2))
                    return;
                if (v1 != null && v2 != null)
                {
                    matrix[v1.Index][v2.Index] = true;
                    matrix[v2.Index][v1.Index] = true;
                    if (!v1.Edges.Contains(v2))
                    {
                        v1.Edges.Add(v2);
                        v2.Edges.Add(v1);
                    }
                }
                else
                {
                    throw new System.ArgumentException("t1 or t2 doesn't exist in the graph");
                }
            }

            public void addEdge(Vertex<T> v1, Vertex<T> v2)
            {
                if (!v1.Equals(v2))
                {
                    if (!v1.Edges.Contains(v2))
                    {
                        v1.Edges.Add(v2);
                        v2.Edges.Add(v1);
                        matrix[v1.Index][v2.Index] = true;
                        matrix[v2.Index][v1.Index] = true;
                    }
                }
            }

            public void deleteEdge(Vertex<T> v1, Vertex<T> v2)
            {
                v1.Edges.Remove(v2);
                v2.Edges.Remove(v1);
                v1.ExtraEdges.Remove(v2);
                v2.ExtraEdges.Remove(v1);
                v1.ExtraEdgeIndex.Remove(v2.Index);
                v2.ExtraEdgeIndex.Remove(v1.Index);
                matrix[v1.Index][v2.Index] = false;
                matrix[v2.Index][v1.Index] = false;
            }

            public List<Vertex<T>> neighbours(Vertex<T> v)
            {
                if (v == null)
                {
                    return new List<Vertex<T>>();
                }
                return v.Edges.ToList();
            }

            public List<Vertex<T>> extraNeighbours(Vertex<T> v)
            {
                return v.ExtraEdges.ToList();
            }

            public int getIndexOfExtraEdge(Vertex<T> v1, Vertex<T> v2)
            {
                if (v1.ExtraEdgeIndex.ContainsKey(v2.Index))
                {
                    return v1.ExtraEdgeIndex[v2.Index];
                }
                return -1;
            }

            public int getIndexOfExtraEdge(T t1, T t2)
            {
                Vertex<T> v1 = GetVertexWithData(t1);
                Vertex<T> v2 = GetVertexWithData(t2);

                return getIndexOfExtraEdge(v1, v2);
            }

            List<Vertex<T>> visited;

            public void dfs(Vertex<T> e, Vertex<T> initial)
            {
                visited.Add(e);
                foreach (Vertex<T> n in neighbours(e))
                {
                    if (n.Equals(initial) && (visited.Count > 2))
                    {
                        checkExistAndAddCycle();
                    }
                    if (!visited.Contains(n))
                    {
                        dfs(n, initial);
                    }
                }
                visited.Remove(e);
            }



            public List<List<Vertex<T>>> GetCycles()
            {
                searchCycles();
                return cycles;
            }

            private void checkExistAndAddCycle()
            {
                bool exist = false;
                foreach (List<Vertex<T>> cycle in cycles)
                {
                    if (samecycle(cycle, visited))
                    {
                        return;
                    }
                }

                List<Vertex<T>> helparr = new List<Vertex<T>>();
                foreach (Vertex<T> v in visited)
                {
                    helparr.Add(v);
                }
                if (!exist)
                    cycles.Add(helparr);
            }

            public void searchCycles()
            {
                visited = new List<Vertex<T>>();
                List<Vertex<T>> s = (List<Vertex<T>>)this.Vertices;
                foreach (Vertex<T> p in s)
                {
                    visited.Clear();
                    dfs(p, p);
                }
            }

            public bool samecycle(List<T> cycle1, List<T> cycle2)
            {
                if (cycle1.Count != cycle2.Count)
                {
                    return false;
                }
                List<Vertex<T>> cycle1v = new List<Vertex<T>>();
                List<Vertex<T>> cycle2v = new List<Vertex<T>>();
                foreach (T i in cycle1)
                {
                    cycle1v.Add(new Vertex<T>(i));
                }
                foreach (T i in cycle2)
                {
                    cycle2v.Add(new Vertex<T>(i));
                }

                return samecycle(cycle1v, cycle2v);
            }

            private static bool samecycle(List<Vertex<T>> cycle1, List<Vertex<T>> cycle2)
            {
                bool result = false;
                if (cycle1.Count != cycle2.Count)
                {
                    return result;
                }
                for (int i = 0; i < cycle1.Count; i++)
                {
                    result = false;
                    if (cycle1.ElementAt(0).Equals(cycle2.ElementAt(i)))
                    {
                        result = true;
                        for (int j = 0; j < cycle1.Count; j++)
                        {
                            if (!cycle1.ElementAt(0 + j).Equals(cycle2.ElementAt((i + j) % cycle1.Count)))
                            {
                                result = false;
                            }
                        }
                        if (result)
                        {
                            return result;
                        }
                        result = true;
                        for (int j = 0; j < cycle1.Count; j++)
                        {
                            if (!cycle1.ElementAt((cycle1.Count - j) % cycle1.Count).Equals(cycle2.ElementAt((i + j) % cycle1.Count)))
                            {
                                result = false;
                            }
                        }
                        if (result)
                        {
                            return result;
                        }
                    }
                }
                return false;
            }

            internal Vertex<T> getFirstRightNeighbourClockwise(Vertex<T> actual, Vertex<T> VertexFromWeCome)
            {

                foreach (EdgeWithAngle v in actual.EdgesWithAngles)
                {
                    if (v.indexEVertex == VertexFromWeCome.Index)
                    {
                        int ind = actual.EdgesWithAngles.IndexOf(v);
                        ind++;
                        ind = ind % actual.EdgesWithAngles.Count;
                        EdgeWithAngle next = actual.EdgesWithAngles.ElementAt(ind);
                        return GetByIndex(next.indexEVertex);
                    }
                }
                return null;
            }

            internal EdgeWithAngle getFirstRightEdgeClockwise(Vertex<T> actual, Vertex<T> VertexFromWeCome)
            {

                foreach (EdgeWithAngle v in actual.EdgesWithAngles)
                {
                    if (v.indexEVertex == VertexFromWeCome.Index)
                    {
                        int ind = actual.EdgesWithAngles.IndexOf(v);
                        ind++;
                        ind = ind % actual.EdgesWithAngles.Count;
                        EdgeWithAngle next = actual.EdgesWithAngles.ElementAt(ind);
                        return next;
                    }
                }
                return null;
            }

            internal EdgeWithAngle GetNextEdgeWithAngle(EdgeWithAngle e0)
            {
                return getFirstRightEdgeClockwise(GetByIndex(e0.indexEVertex), GetByIndex(e0.indexBVertex));
            }

            //najdeme nasledovnu hranu k hrane e0... previous posielame len aby sme ho nemuseli hladat v zozname vrcholov
            internal EdgeWithAngle GetNextEdgeWithAngle(Vertex<T> previous, EdgeWithAngle e0)
            {
                return getFirstRightEdgeClockwise(GetByIndex(e0.indexEVertex), previous);
            }
        }

}


