﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using WindowsFormsApplication1.SNetwork;

namespace WindowsFormsApplication1.MyMath
{
    public class Angles
    {

        public static double RadianToDegree(double angleRadian)
        {
            return angleRadian * (180.0 / Math.PI);
        }

        public static double DegreeToRadian(double angle)
        {
            return Math.PI * angle / 180.0;
        }

        public static bool sameAngles(double angle1,double angle2){
            if (Math.Abs(angle1 - angle2) < Parameters.angleDeviation)
                return true;
            return false;
        }

        public static double AngleToReachPoint(RSensor rs, MyPoint mp)
        {
           
            double angleRS = Angles.angleWithVerticalAxis(rs.b, rs.e, rs.b);
            if (mp.Equals(rs.b))
            {
                throw new Exception("Body sa nemozu rovnat");
            }
            double angleMP = Angles.angleWithVerticalAxis(rs.b, mp, rs.b);

            double angle = (angleMP > angleRS) ? angleMP - angleRS : 360 - angleRS + angleMP;
            if (rs.RotDirection) angle = 360 - angle;

            return angle;
        }


        /// <summary>
        /// Funkcia ráta velkosť uhlu určeného tromi bodmi: (x1,y1),(x2,y2),(x3,y3)
        /// </summary>
        /// <returns>Uhol pri bode (x2,y2)</returns>
        public static double angleTriangle(double x1, double y1, double x2, double y2, double x3, double y3)
        {
            double c = Distances.distance(x1, y1, x2, y2);
            double a = Distances.distance(x2, y2, x3, y3);
            double b = Distances.distance(x1, y1, x3, y3);
            if (a == 0 )
            {
                throw new Exception("distance cannot be 0");
            }

            double cosB = (a * a + c * c - b * b) / (2 * a * c);
            //very important condition... it doesnt workt without this row. Because cosB can be 1.00000000000002 
            //and this is not defined for Acos.
            if (cosB > 1) cosB = 1;
            if (cosB < -1) cosB = -1;
            double B = Math.Acos(cosB);
            if (c * (Math.Sin(B)) != 0 && b==0){
                 throw new Exception("distance cannot be 0");
            }
            double sinC = (c * (Math.Sin(B))) / (b);
            double C = Math.Asin(sinC);

            C = RadianToDegree(C);
            B = RadianToDegree(B);
            C = Math.Round(C);
            if (Double.IsNaN(B))
            {
                throw new Exception("B is not a number");
                //Console.WriteLine("B is not a number");
            }
            return B;
        }

        /// <summary>
        /// Funkcia ráta velkosť uhlu určeného tromi bodmi: p1,p2,p3
        /// </summary>
        /// <returns>Uhol pri bode p2</returns>
        public static double angleTriangle(MyPoint p1, MyPoint p2, MyPoint p3)
        {


            return angleTriangle(p1.x, p1.y, p2.x, p2.y, p3.x, p3.y);
        }

        /// <summary>
        /// vracia uhol aky zoviera vertikalna os vedena smerom hore od axisPoint 
        /// (ktory je jeden z p1,p2) 
        /// a useckov z p1,p2 v smere hod. ruciciek
        /// </summary>
        /// <param name="p1">begin of line</param>
        /// <param name="p2">end of line </param>
        /// <param name="axisPoint">its either p1 or p2</param>
        /// <returns>angle from axis to p1p2 line clockwise</returns>
        public static double angleWithVerticalAxis(MyPoint p1, MyPoint p2, MyPoint axisPoint)
        {
            if (!p1.Equals(axisPoint) && !p2.Equals(axisPoint))
            {
                throw new Exception("Ani jeden bod sa nezhoduje so stredovym bodom");
            }

            MyPoint lastPoint = (p2.Equals(axisPoint)) ? p1 : p2;

            MyPoint firstPoint = new MyPoint(axisPoint.x, axisPoint.y - 10);
            
            if (p1.Equals(p2))
            {
                throw new Exception("nemozu sa rovnat body");
            }

            double angle = Angles.angleTriangle(firstPoint, axisPoint, lastPoint);
            if (axisPoint.x > lastPoint.x)
            {
                angle = 360 - angle;
            }

            return angle;
        }

        internal static double angleToReachVerticalAxis(RSensor rs1)
        {
            double angle = angleWithVerticalAxis(rs1.b, rs1.e, rs1.b);
            if (rs1.RotDirection) angle = 360 - angle;
            return angle;
        }
    }
}
