﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using WindowsFormsApplication1.SNetwork;

namespace WindowsFormsApplication1.MyMath
{
    class Distances
    {

        public static double distance(MyPoint p1, MyPoint p2)
        {
            return distance(p1.x, p1.y, p2.x, p2.y);
        }

        internal static double distance(object p, object p_2)
        {
            if (p is MyPoint && p_2 is MyPoint)
            {
                return distance((MyPoint)p, (MyPoint)p_2);
            }
            return -1;
        }

        public static double distance(double x1, double y1, double x2, double y2)
        {
            double dx = x1 - x2;
            double dy = y1 - y2;
            double d = Math.Sqrt(dx * dx + dy * dy);
            return d;
        }

        public static double distanceSensors(Sensor s1, Sensor s2)
        {
            double vzd1 = distance(s1.b, s2.b);
            double vzd2 = distance(s1.e, s2.b);
            double vzd3 = distance(s1.b, s2.e);
            double vzd4 = distance(s1.e, s2.e);
            double v1 = Math.Min(vzd1, vzd2);
            double v2 = Math.Min(vzd3, vzd4);
            return Math.Min(v1, v2);
        }

        public static double distancePointLineSegment(MyPoint v, MyPoint w, MyPoint p)
        {
            return distancePointLineSegment(new Vector(v.x, v.y), new Vector(w.x, w.y), new Vector(p.x, p.y));
        }


        //http://stackoverflow.com/questions/849211/shortest-distance-between-a-point-and-a-line-segment
        public static double distancePointLineSegment(Vector v, Vector w, Vector p)
        {
            double distSq = v.DistanceToSquared(w); // i.e. |w-v|^2 ... avoid a sqrt
            if (distSq == 0.0) 
            {
                return v.DistanceTo(p);
            }
            // consider the line extending the segment, parameterized as v + t (w - v)
            // We find projection of point p onto the line. 
            // It falls where t = [(p-v) . (w-v)] / |w-v|^2
            double t = (p - v).DotProduct(w - v) / distSq;
            if (t < 0.0) // Beyond the 'v' end of the segment
            {
                return v.DistanceTo(p);
            }
            else if (t > 1.0) // Beyond the 'w' end of the segment
            {
                return w.DistanceTo(p);
            }

            // projection falls on the segment
            Vector projection = v + ((w - v) * t);
            return p.DistanceTo(projection);
        }

        public static double distancePointLine(MyPoint v, MyPoint w, MyPoint p)
        {
            return distancePointLine(new Vector(v.x, v.y), new Vector(w.x, w.y), new Vector(p.x, p.y));
        }

        public static double distancePointLine(Vector v, Vector w, Vector p)
        {
            double distSq = v.DistanceToSquared(w); // i.e. |w-v|^2 ... avoid a sqrt
            if (distSq == 0.0)
            {
                return v.DistanceTo(p);
            }
            // consider the line extending the segment, parameterized as v + t (w - v)
            // We find projection of point p onto the line. 
            // It falls where t = [(p-v) . (w-v)] / |w-v|^2
            double t = (p - v).DotProduct(w - v) / distSq;
            // projection falls on the segment
            Vector projection = v + ((w - v) * t);
            return p.DistanceTo(projection);
        }
    }
}
