﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using WindowsFormsApplication1.SNetwork;
using WindowsFormsApplication1.Events;

namespace WindowsFormsApplication1.MyMath
{
    class IntersectionMath
    {

        public static MyPoint intersectionBeamSensors(Sensor beams1, Sensor beams2)
        {
            double dev = 0.000001;
            double x1 = beams1.b.x, y1 = beams1.b.y, x2 = beams1.e.x, y2 = beams1.e.y,
            x3 = beams2.b.x, y3 = beams2.b.y, x4 = beams2.e.x, y4 = beams2.e.y;

            if (beams1.b.Equals(beams2.b)) return beams1.b;
            if (beams1.e.Equals(beams2.e)) return beams1.e;
            if (beams1.b.Equals(beams2.e)) return beams1.b;
            if (beams1.e.Equals(beams2.b)) return beams1.e;

            double d = (x1 - x2) * (y3 - y4) - (y1 - y2) * (x3 - x4);
            if (d == 0)
            {
                return null;
            }

            //xi a yi je najdeny prienik
            double xi = ((x3 - x4) * (x1 * y2 - y1 * x2) - (x1 - x2)
                    * (x3 * y4 - y3 * x4))
                    / d;
            double yi = ((y3 - y4) * (x1 * y2 - y1 * x2) - (y1 - y2)
                    * (x3 * y4 - y3 * x4))
                    / d;

            //ak xi lezi na prvej usecky a dalej ako je odchylka - null
            if ((xi < x1) && (xi < x2))
            {
                if ((Math.Abs(x1 - xi) > dev) && (Math.Abs(x2 - xi) > dev))
                    return null;
            }
            //ak xi lezi prvej usecky a dalej ako je odchylka - null
            if ((xi > x1) && (xi > x2))
            {
                if ((Math.Abs(xi - x1) > dev) && (Math.Abs(xi - x2) > dev))
                    return null;
            }
            //ak yi lezi prvej usecky a dalej ako je odchylka - null
            if ((yi < y1) && (yi < y2))
            {
                if ((Math.Abs(y1 - yi) > dev) && (Math.Abs(y2 - yi) > dev))
                    return null;
            }
            //ak yi lezi prvej usecky a dalej ako je odchylka - null
            if ((yi > y1) && (yi > y2))
            {
                if ((Math.Abs(yi - y1) > dev) && (Math.Abs(yi - y2) > dev))
                    return null;
            }
            //ak xi lezi druhej usecky a dalej ako je odchylka - null
            if ((xi < x3) && (xi < x4))
            {
                if ((Math.Abs(x3 - xi) > dev) && (Math.Abs(x4 - xi) > dev))
                    return null;
            }
            //ak xi lezi druhej usecky a dalej ako je odchylka - null
            if ((xi > x3) && (xi > x4))
            {
                if ((Math.Abs(xi - x3) > dev) && (Math.Abs(xi - x4) > dev))
                    return null;
            }
            //ak yi lezi druhej usecky a dalej ako je odchylka - null
            if ((yi < y3) && (yi < y4))
            {
                if ((Math.Abs(y3 - yi) > dev) && (Math.Abs(y4 - yi) > dev))
                    return null;
            }
            //ak yi lezi druhej usecky a dalej ako je odchylka - null
            if ((yi > y3) && (yi > y4))
            {
                if ((Math.Abs(yi - y3) > dev) && (Math.Abs(yi - y4) > dev))
                    return null;
            }

            dev = Parameters.pointDeviation;
            //ak je koniec o odchylku od prieniku vratime koniec - da sa odstranit tieto 4 riadky, ak je potrebne
            if (Distances.distance(xi, yi, x1, y1) < (dev)) return beams1.b;
            if (Distances.distance(xi, yi, x2, y2) < (dev)) return beams1.e;
            if (Distances.distance(xi, yi, x3, y3) < (dev)) return beams2.b;
            if (Distances.distance(xi, yi, x4, y4) < (dev)) return beams2.e;
            return new MyPoint(xi, yi);
        }

        public static MyPoint[] intersectionBeamRotSensors(Sensor beams, Sensor rots)
        {
            return intersectionMyLSAndRotSensors(beams, rots); 
        }

        public static MyPoint[] intersectionMyLSAndRotSensors(MyLineSegment ls, Sensor rots)
        {
            double[] line = new double[4];
            line[0] = ls.b.x;
            line[1] = ls.b.y;
            line[2] = ls.e.x;
            line[3] = ls.e.y;

            double[] circle = new double[4];
            circle[0] = rots.b.x;
            circle[1] = rots.b.y;
            circle[2] = rots.e.x;
            circle[3] = rots.e.y;

            return intersectionsLineCircle(line, circle);
        }

        /// <summary>
        /// return all intersections of line and circle: one, two or zero
        /// </summary>
        public static MyPoint[] intersectionsLineCircle(double[] line, double[] circle)
        {
            // make center of circle in [0,0] point
            double x1 = line[0] - circle[0], y1 = line[1] - circle[1];
            double x2 = line[2] - circle[0], y2 = line[3] - circle[1];

            double r = Distances.distance(circle[0], circle[1], circle[2], circle[3]);

            double p1distance = Distances.distance(circle[0], circle[1], line[0],
                    line[1]);
            double p2distance = Distances.distance(circle[0], circle[1], line[2],
                    line[3]);

            double dx = x2 - x1;
            double dy = y2 - y1;
            double dr = Math.Sqrt(dx * dx + dy * dy);
            double D = x1 * y2 - x2 * y1;

            int sgndy = 1;
            if (dy < 0)
            {
                sgndy = -1;
            }
            if ((r * r * dr * dr - D * D) < 0)
            {
                return null;
            }

            double[] result = new double[4];

            result[0] = (D * dy + sgndy * dx * (Math.Sqrt(r * r * dr * dr - D * D)))
                    / (dr * dr);

            result[1] = ((-D) * dx + (Math.Abs(dy))
                    * (Math.Sqrt(r * r * dr * dr - D * D)))
                    / (dr * dr);
            result[2] = (D * dy - sgndy * dx * (Math.Sqrt(r * r * dr * dr - D * D)))
                    / (dr * dr);
            result[3] = ((-D) * dx - (Math.Abs(dy))
                    * (Math.Sqrt(r * r * dr * dr - D * D)))
                    / (dr * dr);

            //vysledok posunieme naspat o tolko ako sme nazaciatku posuvali vsetky body
            result[0] += circle[0];
            result[1] += circle[1];
            result[2] += circle[0];
            result[3] += circle[1];

            MyPoint m1 = new MyPoint(result[0], result[1]);
            MyPoint m2 = new MyPoint(result[2], result[3]);

            if (!(lieOn(m1, line)) && (lieOn(m2, line)))
            {
                return new MyPoint[1] {m2};
            }

            if ((lieOn(m1, line)) && !(lieOn(m2, line)))
            {
                return new MyPoint[1] {m1};
            }

            if ((lieOn(m1, line)) && (lieOn(m2, line)))
            {
                if (m1.Equals(m2))
                    return new MyPoint[1] {m1};
                return new MyPoint[2] {m1,m2};
            }

            return null;
        }

        public static MyPoint Intersection(MyPoint mp1, MyPoint mp2, MyPoint mp3, MyPoint mp4)
        {
            double myconst = Parameters.myconst;
            MyPoint b1 = new MyPoint(mp1.x / myconst,mp1.y);
            MyPoint e1 = new MyPoint(mp2.x / myconst, mp2.y);
            MyPoint b2 = new MyPoint(mp3.x / myconst, mp3.y);
            MyPoint e2 = new MyPoint(mp4.x / myconst, mp4.y);
            return intersectionBeamSensors(new BSensor(b1, e1), new BSensor(b2, e2));
        }


        private static bool lieOn(MyPoint m1, double[] line)
        {
            MyPoint mpLS1   = new MyPoint(line[0],line[1]);
            MyPoint mpLS2   = new MyPoint(line[2],line[3]);


            if (Distances.distancePointLineSegment(mpLS1, mpLS2, m1) < Parameters.pointDeviation)
            {
                return true;
            }
            return false;
        }


    }
}
