﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using WindowsFormsApplication1.SNetwork;
using WindowsFormsApplication1.Events;

namespace WindowsFormsApplication1.MyMath
{
    class OverlapMath
    {


        public static Overlap[] getOverlap(Sensor s1, Sensor s2)
        {
            //zistime typy senzorov a podla toho im vypocitame prekrytie
            if (s1 is RSensor && s2 is RSensor)
            {
                return getOverlap((RSensor)s1, (RSensor)s2);
            }

            if (s1 is BSensor && s2 is RSensor)
            {
                return getOverlap( (RSensor)s2 ,(BSensor)s1);
            }

            if (s1 is RSensor && s2 is BSensor)
            {
                return getOverlap((RSensor)s1, (BSensor)s2);
            }

            if (s1 is BSensor && s2 is BSensor)
            {
                return getOverlap((BSensor)s1, (BSensor)s2);
            }

            return null;
        }

        public static Overlap[] getOverlap(RSensor rs1, RSensor rs2)
        {
            

            rs1.resetPosition();
            rs2.resetPosition();
            Overlap ovl;
            MyPoint[] mpArr;//pomocne pole pre prieniky
            double dev = Parameters.pointDeviation;//epsilon bodu
            //polomery kruznic, ktore opisuju rs1 a rs2
            double lengthRs1 = rs1.getLength();
            double lengthRs2 = rs2.getLength();
            RSensor s;

            //spjime stredy kruznic opisanych rotacnymi senzormi s1 a s5
            MyLineSegment ls = new MyLineSegment(rs1.b, rs2.b);

            //zistime, ci ich vzdialenost stredov kruznic je mensia ako sucet ich polomerov
            if (Distances.distance(rs1.b, rs2.b) < lengthRs2+ lengthRs1 - dev)
            {
                //vypocitame uhly, o ktore sa musia otocit senzory aby dosiahli stred druhej kruznice
                //ale moze sa stat, ze maju rovnako umiestnene stredy - nemozno pocitat uhol

                //ak body zaciatocne=stredy sa rovnaju
                if (ls.getLength() == 0)
                {
                    //ak maju rovnaky smer otacania
                    if (rs1.RotDirection == !rs2.RotDirection)
                    {
                        //pri vypocte uhlu sa nemozu dva body rovnat - inak NaN
                        if (rs1.b.Equals(rs2.e))
                        {
                            throw new Exception("body sa nemozeu rovnat");
                        }
                        //prekryju sa v polovici uhlu, ktory musia k sebe prejst
                        double angleToRS2 = Angles.AngleToReachPoint(rs1, rs2.e);
                        double angle = angleToRS2 / 2;

                        s = (lengthRs1 < lengthRs2) ? rs1 : rs2;
                        s.RotateSensorFromInitial(angle );
                        MyPoint mp = new MyPoint(s.e);
                        s.RotateSensorFromInitial(angle + 180);
                        Overlap ovl1 = new Overlap(new MyPoint(s.b), new MyPoint(s.e), (angle + 180) % 360);
                        ovl1.addActers(rs1, rs2);
                        Overlap ovl2 = new Overlap(new MyPoint(s.b), mp, angle % 360);
                        ovl2.addActers(rs1, rs2);
                        s.resetPosition();
                        return new Overlap[] { ovl1, ovl2 };
                    }
                    else
                    {
                        return null;
                    }
                }

                //pri vypocte uhlu sa nemozu dva body rovnat - inak NaN
                if (rs1.b.Equals(rs2.b))
                {
                    throw new Exception("body sa nemozeu rovnat");
                }
                //pri vypocte uhlu sa nemozu dva body rovnat - inak NaN
                if (rs2.b.Equals(rs1.b))
                {
                    throw new Exception("body sa nemozeu rovnat");
                }
                double angleForRS1 = Angles.AngleToReachPoint(rs1, rs2.b) % 360;
                double angleForRS2 = Angles.AngleToReachPoint(rs2, rs1.b) % 360;
                
                //ak sa tieto uhly zhoduju (s odchylkou epsilon pre ulhy)
                if (Math.Abs(angleForRS1 - angleForRS2) < Parameters.angleDeviation)
                {
                    MyPoint mp1;
                    MyPoint mp2;
                    //vypocitame prienik usecky spajajucej stredy kruznic s kruznicov opisanou s rs1
                    mpArr = IntersectionMath.intersectionMyLSAndRotSensors(ls,rs1);
                    if (mpArr != null)
                    {
                        if (mpArr.Count() > 1)
                        {
                            throw new Exception("Nemoze byt viac ako jeden prienik!");
                        }
                         mp1 = mpArr[0];
                    }
                    else
                    {
                        mp1 = rs2.b;
                    }

                    //vypocitame prienik usecky spajajucej stredy kruznic s kruznicov opisanou s rs2
                    mpArr = IntersectionMath.intersectionMyLSAndRotSensors(ls, rs2);
                    if (mpArr != null)
                    {
                        if (mpArr.Count() > 1)
                        {
                            throw new Exception("Nemoze byt viac ako jeden prienik!");
                        }
                         mp2 = mpArr[0];
                    }
                    else
                    {
                        mp2 = rs1.b;
                    }

                    //vratime prekrytie, ktore je urcene uhlom o ktory sa musia otocit 
                    //a useckou s koncami vypocitanych prienikov
                    ovl = new Overlap(mp1, mp2, angleForRS2 % 360);
                    ovl.addActers(rs1, rs2);
                    return new Overlap[] {ovl};
                }
            }
            return new Overlap[] { null };
        }

        public static Overlap[] getOverlapOld(RSensor rs, BSensor bs)
        {//chyba, ze ked rot. ma dve prekrytia s lucovym
            rs.resetPosition();
            Overlap result = null;
            double lengthRs = Distances.distance(rs.b, rs.e);
            double dev = Parameters.pointDeviation; // epsilon
            //vzdialenosti konc. bodov luc senzoru bs od stredu opisanej kruznice s rs
            double distE = Distances.distance(bs.e, rs.b);
            double distB = Distances.distance(bs.b, rs.b);

            //ci priamka lucovaho senzoru pretina okolie zac. bodu rotacneho senzora
            if (Distances.distancePointLine(bs.b, bs.e, rs.b) < dev)
            {
                //ci je aspon jeden koniec blizko
                if (Distances.distancePointLineSegment(bs.b, bs.e, rs.b) < lengthRs + dev)
                {
                    //vypocitame uhol otocenia rot. senzoru aby nastalo prekrytie
                    MyPoint closer=bs.b;
                    if (distE==0)
                    {
                         closer = bs.b;

                    }
                    else{
                        if (distB == 0)
                        {
                            closer = bs.e;
                        }
                        else
                        {
                            closer = (distE < distB ) ? bs.e : bs.b;
                        }
                    }
                     
                    double angle = Angles.AngleToReachPoint(rs, closer);

                    //ci cely senzor lezi v kruznici
                    if (distB < lengthRs + dev && distE < lengthRs + dev) 
                    {
                        result = new Overlap(bs.b, bs.e, angle % 360);
                    }

                    //len cast senzoru lezi v kruznici
                    else 
                    {
                        //najdeme prienik opisanej kruznice rot. senzorom a useckou luc. senzora, ktory bude koniec prekrytia
                        MyPoint[] inters = IntersectionMath.intersectionBeamRotSensors(bs, rs);
                        if (inters == null || inters.Count() == 2)
                        {
                            throw new Exception("something wrong: there have to be one intersection");
                        }
                        result = new Overlap(closer, inters[0], angle % 360);
                    }
                    //nemoze byt uz result null lebo v obydvoch vetvach if sme ho inicializovali - mozeme pridat akterov
                    result.addActers(rs, bs);
                }
            }

            if (result!= null && result.ls.b.Equals(result.ls.e))
                result = null;
            return new Overlap[] { result};
        }

        public static Overlap[] getOverlap(RSensor rs, BSensor bs)
        {
            MyPoint farther, closer;
            rs.resetPosition();
            List<Overlap> result = new List<Overlap>();
            Overlap result1 = null;
            double angle1 , angle2;
            double lengthRs = Distances.distance(rs.b, rs.e);
            double dev = Parameters.pointDeviation; // epsilon
            //vzdialenosti konc. bodov luc senzoru bs od stredu opisanej kruznice s rs
            double distE = Distances.distance(bs.e, rs.b);
            double distB = Distances.distance(bs.b, rs.b);

            //ci priamka lucovaho senzoru pretina okolie zac. bodu rotacneho senzora
            if (Distances.distancePointLine(bs.b, bs.e, rs.b) < dev)
            {
                //ci je aspon jeden koniec blizko
                if (Distances.distancePointLineSegment(bs.b, bs.e, rs.b) < lengthRs + dev)
                {
                    //vypocitame uhol otocenia rot. senzoru aby nastalo prekrytie

                    closer = (distE < distB) ? bs.e : bs.b;

                    farther = (bs.e.Equals(closer)) ? bs.b : bs.e;


                    if (distE > dev && distB > dev)
                    {
                        angle1 = Angles.AngleToReachPoint(rs, closer);
                        angle2 = Angles.AngleToReachPoint(rs, farther);
                    }
                        //ak je blizsi bod rovny stredu otacania, tak jeho uhol bude uhlom druheho bodu
                    else
                    {
                        
                        angle2 = Angles.AngleToReachPoint(rs, farther);
                        angle1 = angle2;
                    }

                    if (Math.Abs((angle1 % 360) - (angle2 % 360)) < 179)
                    {
                        //ci cely senzor lezi v kruznici
                        if (distB < lengthRs + dev && distE < lengthRs + dev)
                        {
                            result.Add ( new Overlap(bs.b, bs.e, angle1 % 360, rs, bs));
                        }

                        //len cast senzoru lezi v kruznici
                        else
                        {
                            //najdeme prienik opisanej kruznice rot. senzorom a useckou luc. senzora, ktory bude koniec prekrytia
                            MyPoint[] inters = IntersectionMath.intersectionBeamRotSensors(bs, rs);
                            if (inters == null || inters.Count() != 1)
                            {
                                throw new Exception("musi byt prave jeden priesecnik");
                            }
                            result.Add (  new Overlap(closer, inters[0], angle1 % 360, rs, bs));
                        }   
                    }

                    //budu zrejme dve prekrytia
                    else
                    {
                        //ci cely senzor lezi v kruznici
                        if (distB < lengthRs + dev && distE < lengthRs + dev)
                        {
                            if (bs.b.Equals(rs.b) || bs.e.Equals(rs.b))
                            {
                                result1 = new Overlap(bs.b, bs.e, angle1 % 360, rs, bs);
                            }
                            else
                            {

                                result.Add(new Overlap(rs.b, closer, angle1 % 360, rs, bs));
                                result.Add(new Overlap(rs.b, farther, angle2 % 360, rs, bs));
                            }
                        }

                        //len cast senzoru lezi v kruznici
                        else
                        {
                            //najdeme prienik opisanej kruznice rot. senzorom a useckou luc. senzora, ktory bude koniec prekrytia
                            MyPoint[] inters = IntersectionMath.intersectionBeamRotSensors(bs, rs);
                            if (inters == null || inters.Count() == 0)
                            {
                                throw new Exception("musi byt jeden alebo dva priesecniky");
                            }

                            if (inters.Count() == 2)
                            {
                                angle1 = Angles.AngleToReachPoint(rs, inters[0]);
                                angle2 = Angles.AngleToReachPoint(rs, inters[1]);
                                result.Add(new Overlap(rs.b, inters[0], angle1 % 360, rs, bs));
                                result.Add(new Overlap(rs.b, inters[1], angle2 % 360, rs, bs));
                            }

                            if (inters.Count() == 1)
                            {
                                angle2 = Angles.AngleToReachPoint(rs, inters[0]);
                                if (!closer.Equals(rs.b))
                                    result.Add(new Overlap(rs.b, closer, angle1 % 360, rs, bs));
                                result.Add(new Overlap(rs.b, inters[0], angle2 % 360, rs, bs));
                            }
                        }
                    }
                }
            }

            if (result.Count() == 1 && result.First().ls.b.Equals(result.First().ls.e))
                return new Overlap[] { null };

            return result.ToArray() ;



        }

        public static Overlap[] getOverlap(BSensor s1, BSensor s2)
        {
            MyLineSegment ls = getOverlap((MyLineSegment)s1,(MyLineSegment)s2);
            if (ls!=null)
            {
                Overlap ovl = new Overlap(ls, -1);
                ovl.addActers(s1,s2);
                return new Overlap[] { ovl};
            }
            return null;
        }


        public static MyLineSegment getOverlap(MyLineSegment s1, MyLineSegment s2)
        {
            double dev = Parameters.pointDeviation; //epsilon
            bool b1, b2, e1, e2; // urcuju, ci bod lezi na druhej usecke (s presnostou epsilon)

            //vypocitame ci konc. body "lezia" na druhej us. (s presnostou epsilon)
            b1 = Distances.distancePointLineSegment(s2.b, s2.e, s1.b) < dev;
            e1 = Distances.distancePointLineSegment(s2.b, s2.e, s1.e) < dev;
            b2 = Distances.distancePointLineSegment(s1.b, s1.e, s2.b) < dev;
            e2 = Distances.distancePointLineSegment(s1.b, s1.e, s2.e) < dev;

            //pripad ak jedna usecka je podmnoz druhej: oba body jednej usecky lezia na druhej usecke
            if (b1 && e1) return new MyLineSegment(s1.b, s1.e);
            if (b2 && e2) return new MyLineSegment(s2.b, s2.e);

            //pripady ak maju prienik, ale ziadna nie je podmn. duhej a prelrytie je vacsi ako epsilon:

            if (e1 && b2) //zac a koniec prekrytia je e1 a b2
            {
                MyLineSegment ls = new MyLineSegment(s1.e, s2.b);
                if (ls.getLength() > dev) return ls;
            }

            if (e1 && e2) //zac a koniec prekrytia je e1 a e2
            {
                MyLineSegment ls = new MyLineSegment(s1.e, s2.e);
                if (ls.getLength() > dev) return ls;
            }

            if (b1 && b2) //zac a koniec prekrytia je b1 a b2
            {
                MyLineSegment ls = new MyLineSegment(s1.b, s2.b);
                if (ls.getLength() > dev) return ls;
            }

            if (b1 && e2) //zac a koniec prekrytia je b1 a e2
            {
                MyLineSegment ls = new MyLineSegment(s1.b, s2.e);
                if (ls.getLength() > dev) return ls;
            }

            //ziadna z kombinacii nie je splnena: vratime null
            return null;
        }
    }
}
