﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using WindowsFormsApplication1.MyMath;

namespace WindowsFormsApplication1.SNetwork
{
    public class Area : SimpleArea
    {
        public List<SPoint> Out;
        public List<List<SPoint>> In;

        public Area(List<SPoint> V):
            base (V)
        {
        }
    }

    public class SimpleArea
    {
        public int actualIndex= 0;
        public int index = 0;
        public double Angle;
        public List<SPoint> V; 
        public List<SPoint> VCleared;
        public List<SimpleArea> EquivalentSAreas = new List<SimpleArea>(4);
        public bool clear = false;
        public List<SPoint> Polygon;
        public SPoint vyrastok=null;
        public bool inner = false;
        public bool outer = false;
        public List<int> indexesOfSpecEgdes = null;
        public List<int> neighbours = null;
        public SimpleArea samePolygon = null;
        public List<SimpleArea> SpecSubSA = null;

        public SimpleArea(List<SPoint> V ){
            this.V = V;
            //vyhodime take body, ze ked su tri za sebou a lezia na jednom senzore - stredny je zbytocny
            VCleared = getFaceWithoutUselessPoints(V);
            Polygon = getPolygon(V);
            if (vyrastok != null)
            {
                if (Possitions.lieInPolygon(Polygon, vyrastok))
                {
                    inner = true;
                }
                else
                {
                    outer = true;
                }
            }
        }

        private List<SPoint> getPolygon(List<SPoint> VCleared)
        {
            bool founded = true;
            List<SPoint> result = VCleared.ToList();
            SPoint sp1, sp2, sp3;
            //odstranime najskor vyrastky
            while (founded == true && result.Count > 2)
            {
                founded = false;
                int indexOfUsesless = -1;
                for (int i = 0; i < result.Count; i++)
                {
                    sp1 = result.ElementAt(i);
                    sp2 = result.ElementAt((i + 1) % result.Count);
                    sp3 = result.ElementAt((i + 2) % result.Count);

                    indexOfUsesless = (i + 1) % result.Count;

                    //ak lezia vs. tri na jednom senzore a prvy je rovny tretiemu
                    founded = hasCommonOwner(sp1, sp2, sp3) && sp1.Equals(sp3);
                    if (founded)
                    {
                        if (vyrastok == null)
                            vyrastok = sp2;
                        break;
                    }
                    founded = sp1.Equals(sp2);
                    if (founded)
                        break;
                }

                if (founded)
                {
                    result.RemoveAt(indexOfUsesless);
                }
            }

            founded = true;

            while (founded == true && result.Count > 2)
            {
                founded = false;
                int indexOfUsesless = -1;
                for (int i = 0; i < result.Count; i++)               
                {
                    sp1 = result.ElementAt(i);
                    sp2 = result.ElementAt((i + 1) % result.Count);
                    sp3 = result.ElementAt((i + 2) % result.Count);

                    indexOfUsesless = (i + 1) % result.Count;

                    //ak lezia vs. tri na jednom senzore a prvy nie je rovny tretiemu
                    founded = lieOnOneLine(sp1, sp2, sp3);
                    if (founded)
                        break;
                }

                if (founded)
                {
                    
                    result.RemoveAt(indexOfUsesless);
                }
            }
            
            return result;
        }

        private bool lieOnOneLine(SPoint sp1, SPoint sp2, SPoint sp3)
        {
            double dist1 = Distances.distancePointLineSegment(sp1.Location, sp3.Location, sp2.Location);

            if (dist1 < Parameters.pointDeviation) 
                return true;
            return false;
        }

        public SimpleArea(List<SPoint> V, double Angle)
        {
           
            this.V = V;
            this.Angle = Angle;
            VCleared = getFaceWithoutUselessPoints(V);
        }

        //pouzijeme na vypocitavanie ekvivalencii - vyhodime zbytocne SP, ktore mozu zmiznut - rozidenie
        private List<SPoint> getFaceWithoutUselessPoints(List<SPoint> OriginalListSP)
        {
            bool founded = true;
            List<SPoint> result = OriginalListSP.ToList();
            SPoint sp1, sp2, sp3; 
            while (founded == true && result.Count>2)
            {
                founded = false;
                int indexOfUsesless = -1;
                for (int i = 0; i < result.Count; i++)
                {
                    sp1 = result.ElementAt(i);
                    sp2 = result.ElementAt((i + 1) % result.Count);
                    sp3 = result.ElementAt((i + 2) % result.Count);

                    indexOfUsesless = (i + 1) % result.Count;
                    //ak lezia vs. tri na jednom senzore a prvy nie je rovny tretiemu
                    founded = hasCommonOwner(sp1, sp2, sp3) && !sp1.Equals(sp3);
                    if (founded)
                        break;
                }
                if (founded)
                    result.RemoveAt(indexOfUsesless);
            }

            return result;
        }

        private bool hasCommonOwner(SPoint sp1, SPoint sp2, SPoint sp3)
        {
            List<Sensor> list1 = sp1.getCommonOwners(sp2);
            List<Sensor> list2 = sp2.getCommonOwners(sp3);
            List<Sensor> list3 = SPoint.getCommon(list1,list2);

            
            if (list3.Count > 0 )
                return true;
            
            return false;
        }

        public override string ToString()
        {
            String s = "Simple Area: ";
            s = s + "id aktualne:" + this.actualIndex.ToString() + "V: ";
            foreach (SPoint sp in V)
            {
                s = s + sp.GraphIndex.ToString() + ", ";
            }
            return s;
        }

        public void addEqSAreas(List<SimpleArea> sps)
        {
            foreach (SimpleArea sa in sps)
                addEqSArea(sa);
        }

        public void addEqSArea(SimpleArea sa)
        {
            if (!EquivalentSAreas.Contains(sa))
            {
                EquivalentSAreas.Add(sa);
            }
        } 

        internal void computeIndexesOfSpecEgdes(Graph.Graph<SPoint> SpecialGraph)
        {
            SPoint sp1, sp2;
            int index;
            List<int> indexesOfExtraEdges = new List<int>();
            for (int i = 0; i < V.Count; i++)
            {
                sp1 = V.ElementAt(i);
                sp2 = V.ElementAt((i+1) % V.Count);
                index = SpecialGraph.getIndexOfExtraEdge(sp1, sp2);
                if (index != -1)
                    if (!indexesOfExtraEdges.Contains(index) && index != -1)
                        indexesOfExtraEdges.Add(index);
            }
            indexesOfSpecEgdes = indexesOfExtraEdges;
        }

        internal static bool hasCommonExtraEdge(SimpleArea sa1, SimpleArea sa2)
        {
            foreach (int i in sa1.indexesOfSpecEgdes)
            {
                if (sa2.indexesOfSpecEgdes.Contains(i))
                    return true;
            }

            return false;
        }

    }
}
