﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using WindowsFormsApplication1.Events;
using WindowsFormsApplication1.MyMath;
using WindowsFormsApplication1.Graph;

namespace WindowsFormsApplication1.SNetwork
{
    public class Configuration:  IComparable
    {

        public List<Sensor> Sensors = new List<Sensor>(100);
        public double angle = 0;
        public List<Intersection> Intersections;
        public List<Overlap> Overlaps;
        public Graph<SPoint> InductedGraph;
        public List<SimpleArea> SAreas;
        public List<SimpleArea> SpecialSAreas;
        public List<SPoint> SPs;
        public List<SimpleArea> EmptySAreas;
        public List<Event> Events { get; set; }
        public Graph<SPoint> SpecialGraph = null;
        public Graph<SPoint> SpecialGraphReducted = null;
        public List<List<SimpleArea>> Parts = null;
        public List<List<SimpleArea>> SpecialParts = null;

        public Configuration(double angle, List<Intersection> Intersections, List<Overlap> Overlaps, List<Event> Events,  List<Sensor> Sensors)
        {
            this.angle = angle;
            this.Intersections = Intersections.ToList();
            this.Overlaps = Overlaps.ToList();
            this.Sensors = Sensors;
            this.Events = Events;
            this.SPs = ComputeSighnificantPoints();            
            InductedGraph= GraphCreate.makeGraph(this);

            EmptySAreas = SAreasSearch.getEmptyAreas(this);
            InductedGraph.findComponents();

            SAreas = SAreasSearch.searchAreas(InductedGraph);

            Parts = ConfigurationParts.makeParts(SAreas, InductedGraph);

            foreach (SimpleArea sa in SAreas)
                sa.Angle = this.angle;
        }

 



        internal static Configuration makeConfiguration(double e, SNetwork sn, List<Event> allEvents, List<Event> AddPointsOFEvents)
        {
            sn.rotateNetworkAndSearchInters(e);

            if (AddPointsOFEvents != null)
            {
               
                ModifySN.IntegrateEventsInSN(AddPointsOFEvents, sn);
            }

            List<Intersection> actualInters = Intersection.GetListsTogether(sn.IntersectionsStatic, sn.IntersectionsDynamic);

            List<Event> events = EventSearch.getEventsOfAngle(e, allEvents);

            return new Configuration(e, actualInters, sn.getActualOverlaps(), events, sn.Sensors);
        }

        private List<SPoint> ComputeSighnificantPoints()
        {
            List<SPoint> result = new List<SPoint>(Sensors.Count * 3);
            SPoint spE;
            SPoint spB;

            foreach (Sensor s in Sensors)
            {
                List<Sensor> list1 = new List<Sensor>() { s };
                spB = new SPoint(list1, list1, null, angle, s.b);
                spE = new SPoint(list1, null, list1, angle, new MyPoint(s.e.x, s.e.y));
                SPoint spHelp;
                if (result.Contains(spB))
                {
                    spHelp = (result.ElementAt(result.IndexOf(spB)));  
                    spHelp.AddOwner(s);
                    spHelp.AddOwnerBeg(s);
                    spHelp.beginning = true;
                }
                else
                {
                    spB.beginning = true;
                    result.Add(spB);
                }
                if (result.Contains(spE))
                {
                    spHelp = (result.ElementAt(result.IndexOf(spE)));
                    spHelp.end = true;
                    spHelp.AddOwner(s);
                    spHelp.AddOwnerEnd(s);
                }
                else
                {
                    spE.end = true;
                    result.Add(spE);
                }

                foreach (MyPointWithDist Mpwd in s.ActualSPsWithDist)
                {
                    list1 = Mpwd.Acters;
                    spB = new SPoint(list1, null, null, angle, Mpwd.location);
                    if (Mpwd.AddedAsEventLocation)
                        spB.AddedAsEventLocation = true;

                    if (result.Contains(spB))
                    {
                        spHelp = (result.ElementAt(result.IndexOf(spB)));
                        spHelp.AddOwners(Mpwd.Acters);
                        spHelp.inters = true;
                        spHelp.AddedAsEventLocation = spB.AddedAsEventLocation || spHelp.AddedAsEventLocation;
                        spHelp.addForAddEqSPs(Mpwd.ForAddeqSPs);
                    }
                    else
                    {
                        spB.inters = true;
                        result.Add(spB);
                        spB.addForAddEqSPs(Mpwd.ForAddeqSPs);
                    }
                }
            }
            return result;
        }



        public SPoint GetSPWithLoc(MyPoint mp)
        {
            foreach (SPoint sp in SPs)
            {
                if (sp.Location.Equals(mp))
                {
                    return sp;
                }
            }
            return null;
        }

        public String SAreasToString()
        {
            String s = "Oblasti konfiguracie: ";
            if (SAreas != null)
            {
                foreach (SimpleArea sa in SAreas)
                {
                    s = s + "\n\r" + sa.ToString();
                }
            }
            return s;
        }

        public String SpecialSAreasToString()
        {
            String s = "Specialne oblasti konfiguracie: ";
            if (SpecialSAreas != null)
            {
                foreach (SimpleArea sa in SpecialSAreas)
                {
                    s = s + "\n\r" + sa.ToString();
                }
            }
            return s;
        }

        public int CompareTo(object obj)
        {
            if (obj == null) return 1;
            if (!(this is Configuration))
                return 1;

            Configuration s = obj as Configuration;
            if (s.angle > this.angle) return -1;
            return 1;
            
        }

        internal List<SPoint> getCommonSP(Sensor s1, Sensor s2)
        {
            List<SPoint> result = new List<SPoint>(2);
            foreach (SPoint sp in SPs)
            {
                if (sp.Owners.Contains(s1) && sp.Owners.Contains(s2))
                    result.Add(sp);
            }
            return result;  
        }

        internal bool areAllSAreasClear()
        {
            bool result = true;
            foreach(SimpleArea a in this.SAreas){
                if (a.inner)
                    result = result && a.clear;
            }
            return result;
        }

        internal static Configuration getConfWithAllClear(List<Configuration> confs)
        {
            foreach (Configuration conf in confs)
            {
                if (conf.areAllSAreasClear())
                {
                    return conf;
                }
            }
            return null;
        }

    }
}
