﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using WindowsFormsApplication1.MyMath;

namespace WindowsFormsApplication1.SNetwork
{
    public class RSensor : Sensor
    {
        public MyPoint InitialBegin;
        public MyPoint InitialEnd;
        public double DetectAngle;
        public bool RotDirection;
        public static readonly bool CLOCKWISE = true;
        public static readonly bool COUNTERCLOCKWISE = false;
        public static bool PreDefinedDirection = CLOCKWISE;

        public RSensor(MyPoint bT, MyPoint eT, bool direction)
            : base(bT, eT)
        {
            SetFields(bT, eT, direction, 0);
        }

        public RSensor(double x1, double y1, double x2, double y2, bool direction)
        {
            SetFields(new MyPoint(x1, y1), new MyPoint(x2, y2), direction, 0);
        }


        public void RotateSensorFromInitialRadian(double angleRadian)
        {
            resetPosition();
            RotateSensorRadian(angleRadian);
        }

        internal override void RotateSensorFromInitial(double angle)
        {
            resetPosition();
            RotateSensor(angle);
        }


        internal override void RotateSensor(double angle)
        {
            RotateSensorRadian(Angles.DegreeToRadian(angle));
        }

        public void RotateSensorRadian(double angleRadian)
        {
            actualRotAngle = actualRotAngle + Angles.RadianToDegree(angleRadian);
            IntersectionsDynamic.Clear();
            ActualSPsWithDist.Clear();

            double tY = e.y - b.y, tX = e.x - b.x;
            double cosa = Math.Cos(angleRadian);
            double sina = Math.Sin(angleRadian);

            if (RotDirection == false)
            {
                e.x = (tX * cosa) - (tY * sina) + b.x;
                e.y = (tX * sina) + (tY * cosa) + b.y;
            }
            else
            {
                e.x = (tX * cosa) + (tY * sina) + b.x;
                e.y = -(tX * sina) + (tY * cosa) + b.y;
            }
        }



        public  void SetFields(MyPoint bT, MyPoint eT, bool direction, double angle)
        {
            double myconst = Parameters.myconst;
            RotDirection = direction;
            DetectAngle = 0;
            InitialBegin = new MyPoint(bT.x * myconst, bT.y);
            InitialEnd = new MyPoint(eT.x * myconst, eT.y);
            eTriangleNetPoint = eT;
            bTriangleNetPoint = bT;
        }

        public override string ToString()
        {
            return "id=" + this.id + "[center: " + b.ToString() + ",pos: " + e.ToString() + ", rot:" + this.RotDirection.ToString() + "]";
        }

        internal override void resetPosition()
        {
             ActualSPsWithDist.Clear();
           this.e = new MyPoint(InitialEnd);
            this.b = new MyPoint(InitialBegin);
            actualRotAngle = 0;
        }
    }
}
