﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;
using WindowsFormsApplication1.MyMath;
using WindowsFormsApplication1.Events;

namespace WindowsFormsApplication1.SNetwork
{
    public class SNetwork
    {
        public List<Sensor> Sensors = new List<Sensor>(100);
        public double actualAngle = 0;
        public bool changed = true;
        public List<Intersection> IntersectionsStatic = new List<Intersection>(400);
        public List<Intersection> IntersectionsDynamic = new List<Intersection>(400);
        private List<Overlap> Overlaps = new List<Overlap>(100);
        public List<Sensor>[,] SensorsInMash = null;
        public int Period = 0;

        public void clearNetwork()
        {
            this.actualAngle = 0;
            Program.cloud.sn.Overlaps.Clear();
            IntersectionsStatic.Clear();
            IntersectionsDynamic.Clear();
            Sensors.Clear();
        }

        public SNetwork CreateCopy()
        {
            SNetwork copy = new SNetwork();
            copy.Sensors = this.Sensors.ToList();
            copy.rotateNetworkAndSearchInters(this.actualAngle);
            return copy;
        }

        public void SepareteSensorsIntoMash()
        {
            if (Sensors.Count == 0)
                return;
            double maxLength = 0;
            double maxHeight = 0;
            double maxWidth = 0;
            int rows = 0, columns = 0;

            foreach (Sensor s in Sensors)
            {
                maxLength = Math.Max(maxLength, Distances.distance(s.b, s.e));
                maxWidth = Math.Max(maxHeight, s.b.x);
                maxHeight = Math.Max(maxHeight, s.b.y);
            }

            double BoxSize = maxLength * 2;

            double rowsDouble = (maxHeight / BoxSize) + 0.5;
            double columnsDouble = (maxWidth / BoxSize) + 0.5;

            rows = (int)Math.Floor(rowsDouble + 2);
            columns = (int)Math.Floor(columnsDouble + 2); 
            SensorsInMash = new List<Sensor>[columns, rows];


            for (int i = 0; i < columns; i++)
            {
                for (int j = 0; j < rows; j++)
                {
                    SensorsInMash[i, j] = new List<Sensor>();
                }
            }

            foreach (Sensor s in Sensors)
            {
                int row = (int)Math.Floor(s.b.y / BoxSize);
                int column = (int)Math.Floor(s.b.x / BoxSize);
                SensorsInMash[column, row].Add(s);
            }
        }

        public void addOverlap(Overlap ovl)
        {
            if (!this.Overlaps.Contains(ovl))
                this.Overlaps.Add(ovl);
            else
                (this.Overlaps.ElementAt(this.Overlaps.IndexOf(ovl))).addActers(ovl.Acters);//pridame akterov
        }

        public List<Overlap> getActualOverlaps()
        {
            List<Overlap> result = new List<Overlap>(5);
            foreach (Overlap ovl in Overlaps)
            {
                if (Math.Abs(ovl.angle - actualAngle) < 
                  //  0.5)
                    Parameters.angleDeviation   )
                {
                    result.Add(ovl);
                }
            }
            return result;
        }

        public void addIntersectionDynamic(Intersection inters)
        {
            if (!IntersectionsDynamic.Contains(inters))
                IntersectionsDynamic.Add(inters);
            else
            {
                (this.IntersectionsDynamic.ElementAt(this.IntersectionsDynamic.IndexOf(inters))).addActers(inters.Acters);
                (this.IntersectionsDynamic.ElementAt(this.IntersectionsDynamic.IndexOf(inters))).addActersPairs(inters.ActersPairs);            
            }
        }

        public void addIntersectionStatic(Intersection inters)
        {
            if (!IntersectionsStatic.Contains(inters))
                IntersectionsStatic.Add(inters);
            else
            {
                (this.IntersectionsStatic.ElementAt(this.IntersectionsStatic.IndexOf(inters))).addActers(inters.Acters);
                (this.IntersectionsStatic.ElementAt(this.IntersectionsStatic.IndexOf(inters))).addActersPairs(inters.ActersPairs);
            }
        } 

        public void findIntersections()
        {

            foreach (Sensor s in Sensors)
            {
                s.ActualSPsWithDist.Clear();
            }
            if (this.changed)
            {
                
                changed = false;
                SepareteSensorsIntoMash();
                this.findOverlaps();
                IntesectionsSearch.searchStaticIntersections(this);
               // Program.cloud.WriteToLog("Prepocital som staticke pries. a prekrytia");
            }
            IntesectionsSearch.searchDynamicIntersections(this);
            IntesectionsSearch.sortIntersections(Sensors);
        }


        /// <summary>
        /// Vymazeme senzor zadanim stplcov a riadkov, v ktorych sa nachadzaju jeho koncove body v akutalnej polohe
        /// </summary>
        /// <param name="p1">jeden koniec senzora (stlpec, riadok)</param>
        /// <param name="p2">druhy koniec senzora (stlpec, riadok) </param>
        internal bool deleteSensorWithActualPosition(MyPoint p1, MyPoint p2)
        {
            bool res = false;
            List<Sensor> DeleteTo = new List<Sensor>();
            p1.x = p1.x * Parameters.myconst;
            p2.x = p2.x * Parameters.myconst;
            foreach (Sensor s in this.Sensors)
            {
                if ((s.b.Equals(p1) && s.e.Equals(p2)) || (s.b.Equals(p2) && s.e.Equals(p1)))
                {
                    DeleteTo.Add(s);
                    this.changed = true;
                }
            }
            res = (DeleteTo.Count() > 0);
            foreach (Sensor s in DeleteTo)
                Sensors.Remove(s);
            DeleteTo.Clear();
            return res;
        }

        public void AddSensor(Sensor s){
            if (!Sensors.Contains(s))
            {
                Sensors.Add(s);
                changed = true;
            }
            else {
                Console.WriteLine("sensor uz existuje, nebude pridany!!");
            }
        }



        public void resetActualNetwork()
        {
            actualAngle = 0;
            IntersectionsDynamic.Clear();
            foreach (Sensor s in Sensors)
            {
                
                if (s is RSensor)
                    ((RSensor)s).resetPosition();
                
                    s.resetPosition();

            }
        }

        public void rotateNetworkAndSearchInters(double angle)
        {
            
            rotateNetwork(angle);
            findIntersections();
        }

        public void rotateNetwork(double angle)
        {
            if (angle < 0)
                return;

            this.Period = (int) Math.Floor( angle / 360);
            resetActualNetwork();
            actualAngle = angle % 360;
            foreach (Sensor s in Sensors)
            {
                if (s is RSensor) 
                    ((RSensor)s).RotateSensor(actualAngle);
                if (s is BSensor) 
                    ((BSensor)s).RotateSensor(actualAngle);

                if (s is BSensor)
                {
                    ((BSensor)s).IntersectionsDynamic.Clear();
                    ((BSensor)s).ActualSPsWithDist.Clear();
                }
            }
        }


        internal void findOverlaps()
        {
            double angle = this.actualAngle;
            OverlapsSearch.searchOverlaps(this);
            this.rotateNetwork(angle);
        }

        internal void OverlapsClear()
        {
            Overlaps.Clear();
        }

        public override String ToString()
        {
            String result ;

            result = "_____________________________________ \r\n  Aktualna siet senzorov:";
           
            foreach (Sensor s in Sensors)
            {
                result = result + "\r\n  " + s.ToString();
            }
            result = result + "\r\n"+ "_____________________________________";
           // result = result + "\r\n" + OverlapsToSTring();
            return result;
        }

        public String OverlapsToSTring()
        {
            String result;

            result = "_____________________________________ \r\n  Prekrytia siete senzorov:";

            foreach (Overlap s in Overlaps)
            {
                result = result + "\r\n  " + s.ToString();
            }
            result = result + "\r\n"  + "_____________________________________";
            return result;
        }

        internal List<Sensor> getCloseSensors(int i, int j)
        {
            //vratime vs. senzory v susednych boxoch
            List<Sensor> result = new List<Sensor>(20);
            //ak sme v rozsahu pola
            if (i>=0 && j>=0 && i < SensorsInMash.GetLength(0) && j < SensorsInMash.GetLength(1))
            {
                //pridame vs. prvky v tom danom boxe
                result = SensorsInMash[i, j].ToList();
                //ak nie sme v 0tom stlpci vratime prvky laveho boxu
                if (i > 0)
                {
                    foreach (Sensor s in SensorsInMash[i - 1, j])
                        result.Add(s);
                }
                //ak nie sme v 0tom riadku vratime prvky vyssieho boxu
                if (j > 0)
                {
                    foreach (Sensor s in SensorsInMash[i , j - 1])
                        result.Add(s);
                }
                //ak ex. box napravo: pocet stpcov je viac ako aktualny stlpec +1
                if ((i +1) < SensorsInMash.GetLength(0) )
                {
                    foreach (Sensor s in SensorsInMash[i + 1, j])
                        result.Add(s);
                }
                //ak ex. box pod nim: pocet riadkov je viac ako aktualny +1
                if ((j + 1) < SensorsInMash.GetLength(1))
                {
                    foreach (Sensor s in SensorsInMash[i, j + 1])
                        result.Add(s);
                }

                //ak ex. stlpec NAPRAVO a zaroven riadok POD akutalnym boxom
                if ((i + 1) < SensorsInMash.GetLength(0) && (j + 1) < SensorsInMash.GetLength(1))
                {
                    foreach (Sensor s in SensorsInMash[i +1, j + 1])
                        result.Add(s);
                }
                //ak ex. stlpec NAPRAVO a zaroven riadok NAD akutalnym boxom
                if ((i + 1) < SensorsInMash.GetLength(0) && j > 0)
                {
                    foreach (Sensor s in SensorsInMash[i + 1, j - 1])
                        result.Add(s);
                }
                //ak ex. stlpec NALAVO a zaroven riadok POD akutalnym boxom
                if (i > 0 && (j + 1) < SensorsInMash.GetLength(1))
                {
                    foreach (Sensor s in SensorsInMash[i - 1, j + 1])
                        result.Add(s);
                }
                //ak ex. stlpec NALAVO a zaroven riadok NAD akutalnym boxom
                if (i > 0 && j > 0)
                {
                    foreach (Sensor s in SensorsInMash[i - 1, j - 1])
                        result.Add(s);
                }
            }
            return result;
        }
    }
}
