﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using WindowsFormsApplication1.Events;
using WindowsFormsApplication1.MyMath;

namespace WindowsFormsApplication1.SNetwork
{
    public class Sensor : MyLineSegment, IComparable
    {
        public int id;
        public static int maxid = 0;
        public List<Intersection> IntersectionsDynamic = new List<Intersection>(10);
        public List<Intersection> IntersectionsStatic = new List<Intersection>(10);
        public List<MyPointWithDist> ActualSPsWithDist = new List<MyPointWithDist>(10);

        protected List<Overlap> Overlaps = new List<Overlap>(10);
        public MyPoint bTriangleNetPoint = null;
        public MyPoint eTriangleNetPoint = null;
        public double actualRotAngle = 0;



        public Sensor(MyPoint b, MyPoint e)
            : base(b, e)
        {
            double myconst = Parameters.myconst;
            id = maxid;
            maxid++;
            bTriangleNetPoint = b;
            eTriangleNetPoint = e;
            this.b = new MyPoint(b.x * myconst, b.y);
            this.e = new MyPoint(e.x * myconst, e.y);
        }

        public Sensor()
        {
            id = maxid;
            maxid++;
        }

        public override string ToString()
        {
            return "[b: " + b.ToString() + ",e: " + e.ToString() + "]";
        }

        public override bool Equals(Object obj)
        {
            if (!(obj is Sensor)) return false;
            Sensor s = (Sensor)obj;
            if (s.id == this.id)
                return true;
            return false;
        }

        public override int GetHashCode()
        {
            return base.GetHashCode();
        }

        public int CompareTo(object obj)
        {
            if (obj == null) return 1;

            Sensor s = obj as Sensor;

            if (this is RSensor)
            {
                if (s is RSensor) return 0;
                if (s is BSensor) return 1;
            }
            if (this is BSensor)
            {
                if (s is RSensor) return -1;
                if (s is BSensor) return 0;
            }
            return 1;
        }

        public static bool isClose(Sensor s1, Sensor s2)
        {
            if ((int)Distances.distanceSensors(s1, s2) < Math.Max(Distances.distance(s1.b, s1.e), Distances.distance(s2.b, s2.e))) return true;
            else return false;
        }

        internal bool hasOverlapWith(Sensor s2)
        {
            foreach (Overlap ovl in this.getActualOverlaps())
            {
                if (ovl.Acters.Contains(s2))
                    return true;
            }
            return false;
        }

        public void addIntersectionDynamic(Intersection inters){
            if (!IntersectionsDynamic.Contains(inters))
                IntersectionsDynamic.Add(inters);
            else
            {
                (this.IntersectionsDynamic.ElementAt(this.IntersectionsDynamic.IndexOf(inters))).addActers(inters.Acters);
                (this.IntersectionsDynamic.ElementAt(this.IntersectionsDynamic.IndexOf(inters))).addActersPairs(inters.ActersPairs);            
            }
        }

        public void addIntersectionStatic(Intersection inters)
        {
            if (!IntersectionsStatic.Contains(inters))
                IntersectionsStatic.Add(inters);
            else
            {
                (this.IntersectionsStatic.ElementAt(this.IntersectionsStatic.IndexOf(inters))).addActers(inters.Acters);
                (this.IntersectionsStatic.ElementAt(this.IntersectionsStatic.IndexOf(inters))).addActersPairs(inters.ActersPairs);
            }
        }

        public void addOverlap(Overlap ovl)
        {
            if (ovl != null){
                if (!this.Overlaps.Contains(ovl))
                    this.Overlaps.Add(ovl);
                else
                    (this.Overlaps.ElementAt(this.Overlaps.IndexOf(ovl))).addActers(ovl.Acters);//pridame akterov
            }
        }

        public void addMyPointWithDist(MyPointWithDist mp)
        {
            if (mp != null)
            {
                if (!this.ActualSPsWithDist.Contains(mp))
                {
                    this.ActualSPsWithDist.Add(mp);
                }
                else
                {
                    MyPointWithDist mpE = (this.ActualSPsWithDist.ElementAt(this.ActualSPsWithDist.IndexOf(mp)));
                    mpE.addActers(mp.Acters);//pridame akterov
                }
            }
        }


        public List<Overlap> getActualOverlaps()
        {
            List<Overlap> result = new List<Overlap>(5);
            foreach (Overlap ovl in Overlaps)
            {
                if (Math.Abs(ovl.angle - actualRotAngle ) < Parameters.angleDeviation)
                {
                    result.Add(ovl);
                }
            }
            return result;
        }

        internal virtual void resetPosition()
        {
            IntersectionsDynamic.Clear();
            ActualSPsWithDist.Clear();
           
        }

        internal virtual void RotateSensorFromInitial(double angle)
        {
            resetPosition();
            RotateSensor(angle);
        }


        internal virtual void RotateSensor(double actualAngle)
        {
            actualRotAngle = actualAngle;
        }
    }
}
