﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using WindowsFormsApplication1.Events;
using WindowsFormsApplication1.MyMath;

namespace WindowsFormsApplication1.SNetwork
{
    class Step
    {
        List<Intersection> IntersectionsStatic;
        List<Intersection> IntersectionsDynamic;
        List<Intersection> Intersections = new List<Intersection>(200);

        Dictionary<int, List<MyPointWithDist>> SensorsActualSPs = new Dictionary<int, List<MyPointWithDist>>();

        List<Overlap> Overlaps = new List<Overlap>(200);
        List<Sensor> SensorsAll;
        public double angle;

        public static Step makeStep(double angle)
        {
             
            SNetwork sn = Program.cloud.sn;
            return new Step(sn.IntersectionsStatic, sn.IntersectionsDynamic, sn.getActualOverlaps(), angle);
        }


        public override string ToString()
        {
            return "Step ; uhol: " + angle.ToString();
        }

        //uhol je aj vacsi ako 360 - potrebujeme to pre lahsie urcovanie, ktory step nasleduje po ktorom, pre urcenie udalosti cez hranicu 360
        public Step(List<Intersection> IntersectionsStatic, List<Intersection> IntersectionsDynamic, List<Overlap> Overlaps, double angle)
        {
            this.IntersectionsStatic = IntersectionsStatic.ToList();
            this.IntersectionsDynamic = IntersectionsDynamic.ToList();
            this.angle = angle;
            SensorsAll = Program.cloud.sn.Sensors;
            this.Overlaps = Overlaps.ToList();
            
            getIntersectionsTogether();

            foreach (Sensor s in Program.cloud.sn.Sensors)
            {
                SensorsActualSPs.Add(s.id, s.ActualSPsWithDist.ToList());
            }
        }

        private void getIntersectionsTogether()
        {
            foreach (Intersection inters in IntersectionsDynamic)
            {
                processInters(inters);
            }
            foreach (Intersection inters in IntersectionsStatic)
            {
                processInters(inters);
            }
        }

        private void processInters(Intersection inters)
        {
            if (Intersections.Contains(inters)) // uz je prienik ulozeny
            {
                foreach (Sensor s in inters.Acters) // pridame novych akterov
                {
                    Intersections.ElementAt(Intersections.IndexOf(inters)).addActer(s);
                }
            }
            else // nemame tento prienik este
            {
                Intersections.Add(inters); // prienik pridame
            }
        }



        private MyPoint getIntersection(Sensor s1, Sensor s2)
        {
            foreach (Intersection inters in Intersections)
            {
                if (inters.Acters.Contains(s1) && inters.Acters.Contains(s2) )
                    return inters.mp;
            }
            return null;
        }

        private bool gotIntersection(Sensor s1, Sensor s2)
        {
            if (getIntersection(s1, s2) != null) return true;
            return false;
        }

        private MyLineSegment getOverlap(Sensor s1, Sensor s2)
        {
            foreach (Overlap ovl in Overlaps)
            {
                if (ovl.Acters.Contains(s1) && ovl.Acters.Contains(s2))
                    return ovl.ls;
            }
            return null;
            //throw new NotImplementedException();
        }

        public List<Event> getEvents(Step NextSetp)
        {
            //takto si budeme drzat nase kroky na lahsiu pracu s nimi (na poradi zalezi)
            Step First;
            Step Secound;

            //urcime, ktory z krokov je predchadzajuci a ktory nasledovny...
            //predpokladame, ze sa nebudeme pytat na kroky cez hranicu 360
            if (this.angle < NextSetp.angle)
            {
                First = this;
                Secound = NextSetp;
            }
            else
            {
                First = NextSetp;
                Secound = this;
            }

            //pomocny objekt Event
            Event ev = null;
            //lokalizacie bodu
            MyPoint FirstLocation, SecoundLocation;
            //vsetky najdene eventy
            List<Event> Events = new List<Event>(30);
            //prekrytia, ktore sa budu zdielat dva senzory v First a v Secound kroku
            MyLineSegment Overlap1, Overlap2;
          

            
            
            //prejdeme vsetky dvojice senzorov a budeme zistovat, ci nie je udalost medzi nimi
            foreach (Sensor s1 in this.SensorsAll)
            {
                foreach (Sensor s2 in this.SensorsAll)
                {
                    //zistujeme to iba ak s1!=s2
                    if (!s1.Equals(s2))
                    {
                        //najdeme prekrytia tychto senzorov v krokoch First a Secound
                        Overlap1 = First.getOverlap(s1, s2);
                        Overlap2 = Secound.getOverlap(s1, s2);

                        //ak najdeme nejake prekrytie vratime to ako udalsot prekrytia
                        if (Overlap1 != null || Overlap2 != null)
                        {
                            ev = null;
                            //pridame prieniky, ako udalosti
                            if (Overlap1 != null)
                                ev = new Event(First.angle % 360, Event.OVERLAP, Overlap1.b);
                            if (Overlap2 != null)
                                ev = new Event(Secound.angle % 360, Event.OVERLAP, Overlap2.b);

                            //ak tento event este nemame zaznaceny, spravime tak
                            if (!Events.Contains(ev))
                                Events.Add(ev);

                            //pridame mu akterov, aj dvojicu akterov
                            Events.ElementAt(Events.IndexOf(ev)).addActers(s1, s2);
                            Events.ElementAt(Events.IndexOf(ev)).addActersPair(new PairOfActers(s1, s2));
                        }


                        //najdeme spolocne prisecniky v krokoch First a Secound
                        FirstLocation = First.getIntersection(s1, s2);
                        SecoundLocation = Secound.getIntersection(s1, s2);

                        //ak existuje prienik v prvom kroku a vdruhom nie a ani prekrytie v druhom nie je
                        if (FirstLocation != null && SecoundLocation == null && Overlap2 == null)
                        {
                            //nastava udalost rozidenia senzorov v uhle prveho kroku
                            ev = new Event(First.angle % 360, Event.SEPARATION, FirstLocation);
                            
                            //ak tento event este nemame zaznaceny, spravime tak
                            if (!Events.Contains(ev))
                                Events.Add(ev);

                            //pridame mu akterov, aj dvojicu akterov
                            Events.ElementAt(Events.IndexOf(ev)).addActers(s1, s2);
                            Events.ElementAt(Events.IndexOf(ev)).addActersPair(new PairOfActers(s1, s2));
                        }

                        //ak existuje prienik v druhom a kroku a v prvom nie (ani prekrytie)
                        if (FirstLocation == null && SecoundLocation != null && Overlap1 == null)
                        {
                            //nastava udalost stretnutia v uhle druheho kroku
                            ev = new Event(Secound.angle % 360, Event.MEET, SecoundLocation);

                            //ak tento event este nemame zaznaceny, spravime tak
                            if (!Events.Contains(ev))
                                Events.Add(ev);

                            //pridame mu akterov, aj dvojicu akterov
                            Events.ElementAt(Events.IndexOf(ev)).addActers(s1, s2);
                            Events.ElementAt(Events.IndexOf(ev)).addActersPair(new PairOfActers(s1, s2));
                        }

                    }
                }
            }

            List<WithActers> checkedS= new List<WithActers>(10);
            MyPoint FirstL1;
           // Sensor rs,bs;
            List<Sensor> checkedSensor = new List<Sensor>(Program.cloud.sn.Sensors.Count);

            List<Sensor>[,] sensorsInMash = Program.cloud.sn.SensorsInMash;

            //najdeme trojice, ktore maju spolocne body
            if (sensorsInMash == null )
            {
                foreach (Sensor s1 in SensorsAll)
                {
                    if ((s1 is RSensor)) //&&  //s1.IntersectionsDynamic.Count > 0)
                    {
                        foreach (Sensor s2 in SensorsAll)
                        {
                           

                            //ak sa nerovnaju
                            if (!s1.Equals(s2))
                                //ich priesecnik vyopcitame
                                FirstL1 = First.getIntersection(s1, s2);
                            else
                                //inak ho dame za null
                                FirstL1 = null;

                            //ak sme nasli priesecnik s1 a s2
                            if (FirstL1 != null)
                            {
                                //prejdeme senzory
                                foreach (Sensor s3 in SensorsAll)
                                {
                                    //ak sme tuto trojicu este nekontrolovali a neex. event s touto trojicou akterov
                                    if (!existWithActers(s1, s2, s3, Events) && !existWithActers(s1, s2, s3, checkedS))
                                    //skontrolujeme trojicu
                                    {
                                        WithActers wa = new WithActers();
                                        wa.addActers(s1, s2);
                                        wa.addActer(s3);
                                        checkedS.Add(wa);

                                        checkthreeSensors(s1, s2, s3, First, Secound, FirstL1, Events);

                                    }
                                }
                            }
                        }
                    }
                }
            }

            else
            {
                List<  Sensor> closeSensors;

                for (int i = 0; i < sensorsInMash.GetLength(0); i++)
                {
                    for (int j = 0; j < sensorsInMash.GetLength(1); j++)
                    {
                        foreach (Sensor s1 in sensorsInMash[i, j])
                        {
                            if (!checkedSensor.Contains(s1))
                            {
                                if ((s1 is RSensor))
                                {
                                    closeSensors = Program.cloud.sn.getCloseSensors(i, j);

                                    foreach (Sensor s2 in closeSensors)
                                    {
                                        if (!checkedSensor.Contains(s2))
                                        {
                                            //ak sa nerovnaju
                                            if (!s1.Equals(s2))
                                                //ich priesecnik vyopcitame
                                                FirstL1 = First.getIntersection(s1, s2);
                                            else
                                                //inak ho dame za null
                                                FirstL1 = null;

                                            //ak sme nasli priesecnik s1 a s2
                                            if (FirstL1 != null)
                                            {
                                                //prejdeme senzory
                                                foreach (Sensor s3 in closeSensors)
                                                {
                                                    if (!checkedSensor.Contains(s3) && (!s3.Equals(s2)) && (!s1.Equals(s3)) )
                                                    {
                                                        //ak sme tuto trojicu este nekontrolovali a neex. event s touto trojicou akterov
                                                        if (!existWithActers(s1, s2, s3, Events) && !existWithActers(s1, s2, s3, checkedS))
                                                        {
                                                            //skontrolujeme trojicu
                                                            WithActers wa = new WithActers();
                                                            wa.addActers(s1, s2);
                                                            wa.addActer(s3);
                                                            checkedS.Add(wa);

                                                            if (getOverlap(s1, s2) == null && getOverlap(s1, s3) == null && getOverlap(s3, s2) == null)
                                                                checkthreeSensors(s1, s2, s3, First, Secound, FirstL1, Events);

                                                        }
                                                    }
                                                }
                                            }
                                        }
                                    }
                                    checkedSensor.Add(s1);
                                }
                            }   
                        }
                    }
                }
            }
            return Events;
        }

        private void checkthreeSensors(Sensor s1, Sensor s2, Sensor s3, Step First, Step Secound, MyPoint FirstL1, List<Event> Events)
        {
            if (s2 is BSensor && s3 is RSensor)
            {
                RSensor rs = (RSensor) s3;
                s3 = s2;
                s2 = rs;
            }


            MyPoint  FirstL2, FirstL3;
            MyPoint SecoundL1, SecoundL2, SecoundL3;
            Event ev = null;

            if (!s2.Equals(s3) && !s1.Equals(s3))
            {
               // FirstL1 = First.getIntersection(s2, s1);
                FirstL2 = First.getIntersection(s2, s3);
                FirstL3 = First.getIntersection(s1, s3);
            }
            else
            {
                FirstL1 = null;
                FirstL2 = null;
                FirstL3 = null;
            }
            //nasli sme tri, ktore maju spolocne tri body 
            if (FirstL1 != null && FirstL2 != null && FirstL3 != null)
            {
                //zistime, ci sa nestretnu
                SecoundL1 = Secound.getIntersection(s1, s2);
                SecoundL2 = Secound.getIntersection(s2, s3);
                SecoundL3 = Secound.getIntersection(s1, s3);

                if (SecoundL1 == null || SecoundL2 == null || SecoundL3 == null)
                    return;

                bool Eq1 = FirstL1.Equals(FirstL2) || FirstL1.Equals(FirstL3) || FirstL2.Equals(FirstL3);
                bool Eq2 = SecoundL1.Equals(SecoundL2) && SecoundL1.Equals(SecoundL3) && SecoundL2.Equals(SecoundL3);
                bool Eq3 = !FirstL1.Equals(FirstL2) || !FirstL1.Equals(FirstL3) || !FirstL2.Equals(FirstL3);
                //ak su v dalsom kroku tieto body uz spojene
                /*moze sa stat, ked su dva prekryvajuce sa, ze prienik berie ako jeden koniec prekrytia a po otoceni bude prienikom druhy koniec prekrytia -> zrazu maju spolocny bod traja*/
                if (Eq3 && Eq2)
                {
                    ev = new Event(Secound.angle , 0, FirstL2);
                    ev.addActer(s1);
                    ev.addActers(s2, s3);
                    if (!Events.Contains(ev))
                        Events.Add(ev);
                }

                //ak nie su ziadne dva rovnake
                if (Eq3 && !Eq2)
                {
                    //ak je jeden RS a dva BS
                    if ((s2 is BSensor) && (s3 is BSensor))
                    {
                        ((RSensor)s1).RotateSensorFromInitial(First.angle);
                        double toFirst1 = Angles.AngleToReachPoint((RSensor)s1, FirstL2);
                        //ak nastane stretnutie skor ako je nasled pseudo krok
                        if (toFirst1 < (Secound.angle - First.angle) + Parameters.angleDeviation)
                        {
                            ev = new Event(First.angle + toFirst1, 0, FirstL2);
                            ev.addActer(s1);
                            ev.addActers(s2, s3);
                            if (!Events.Contains(ev))
                                Events.Add(ev);
                        }
                    }

                    if (s3 is BSensor && s2 is RSensor)
                    {
                        ((RSensor)s2).RotateSensorFromInitial(First.angle);
                        ((RSensor)s1).RotateSensorFromInitial(First.angle);
                        double toFirst1 = Angles.AngleToReachPoint((RSensor)s1, FirstL2);
                        double toFirst2 = Angles.AngleToReachPoint((RSensor)s2, FirstL3);

                        if (toFirst1 > 3 * (Secound.angle - First.angle) || toFirst2 > 3 * (Secound.angle - First.angle))
                            return;
                        
                        double dist = Distances.distance(FirstL2, FirstL3);
                        double rate1 = dist / toFirst1;
                        double rate2 = dist / toFirst2;
                        double toMeeting = dist / (rate1 + rate2);
                        if (toMeeting < (Secound.angle - First.angle) + Parameters.angleDeviation)
                        //pridat akterov
                        {
                            ev = new Event(First.angle + toMeeting, 0, new MyPoint(0, 0));
                            ev.addActer(s1);
                            ev.addActers(s2, s3);
                            if (!Events.Contains(ev))
                                Events.Add(ev);
                        }
                    }


                    if (s3 is RSensor && s2 is RSensor)
                    {
                        ((RSensor)s3).RotateSensorFromInitial(First.angle);
                        ((RSensor)s2).RotateSensorFromInitial(First.angle);
                        ((RSensor)s1).RotateSensorFromInitial(First.angle);
                        double toFirst1 = Angles.AngleToReachPoint((RSensor)s1, FirstL2);
                        double toFirst2 = Angles.AngleToReachPoint((RSensor)s2, FirstL3);
                        double toFirst3 = Angles.AngleToReachPoint((RSensor)s3, FirstL1);

                        double dist1 = Distances.distance(FirstL2, FirstL3);
                        double dist2 = Distances.distance(FirstL1, FirstL3);
                        double dist3 = Distances.distance(FirstL1, FirstL2);

                        double rate11 = dist1 / toFirst1;
                        double rate12 = dist1 / toFirst2;

                        double rate22 = dist2 / toFirst2;
                        double rate23 = dist2 / toFirst3;

                        double rate33 = dist3 / toFirst3;
                        double rate31 = dist3 / toFirst1;

                        double toMeeting1 = dist1 / (rate11 + rate12 + rate23);
                        double toMeeting2 = dist2 / (rate22 + rate23 + rate11);
                        double toMeeting3 = dist3 / (rate31 + rate33 + rate12);

                        double stepsGap = (Secound.angle - First.angle);

                        double min = Math.Min(Math.Min(toMeeting1, toMeeting2), toMeeting3);
                        double max = Math.Max(Math.Max(toMeeting1, toMeeting2), toMeeting3);
                        if ((min + max) / 2 <= stepsGap)
                        //pridat akterov
                        {
                            ev = new Event(First.angle + (min + max) / 2, 0, new MyPoint(0, 0));
                            ev.addActer(s1);
                            ev.addActers(s2, s3);
                            if (!Events.Contains(ev))
                                Events.Add(ev);
                        }
                    }
                }
            }
        }



        private bool existWithActers(Sensor s1, Sensor s2, Sensor s3, List<Event> Events)
        {
            foreach (Event ev in Events)
            {
                if (ev.type == 0)
                {
                    if (ev.Acters.Contains(s1) && ev.Acters.Contains(s2) && ev.Acters.Contains(s3))
                    {
                        return true;
                    }
                }
            }
            return false;
        }

        private bool existWithActers(Sensor s1, Sensor s2, Sensor s3, List<WithActers> WithActers)
        {
            foreach (WithActers ev in WithActers)
            {
                    if (ev.Acters.Contains(s1) && ev.Acters.Contains(s2) && ev.Acters.Contains(s3))
                    {
                        return true;
                    }
            }
            return false;
        }

        private List<Sensor> SameOrderOfSPs(Step First, Step Secound)
        {
            List<Sensor> result = new List<Sensor>(5);
            foreach (Sensor s in First.SensorsAll)
            {              
                List<MyPointWithDist> FirstListmpwd =   First.SensorsActualSPs[s.id];
                List<MyPointWithDist> SecoundListmpwd = Secound.SensorsActualSPs[s.id];
                //pripad ak maju rozne pocty nas nezuajima
          
                if ( !EqualsOrderInLists(FirstListmpwd, SecoundListmpwd) ) result.Add( s);

            }
            if (result.Count > 0) return result;
            return null;
        }
        /// <summary>
        /// len ak su rovnako velke tieto zoznami a maju rozne poradie vratime false... 
        /// ak nemaju rovnako vela prvkou tak vratime true 
        /// </summary>
        /// <param name="FirstListmpwd"></param>
        /// <param name="SecoundListmpwd"></param>
        /// <returns></returns>
        private bool EqualsOrderInLists(List<MyPointWithDist> FirstListmpwd, List<MyPointWithDist> SecoundListmpwd)
        {
            if ( FirstListmpwd.Count != SecoundListmpwd.Count) return false;
           // if (FirstListmpwd.Count < 3) return false;
            for (int i = 0; i < FirstListmpwd.Count; i++)
            {
                WithActers wa1 = (WithActers)FirstListmpwd.ElementAt(i);
                WithActers wa2 = (WithActers)SecoundListmpwd.ElementAt(i);
                
                if (wa1.Acters.Count != wa2.Acters.Count)
                    return false;
            
                foreach (Sensor s in wa1.Acters)
                {
                    if (!wa2.Acters.Contains(s))
                        return false;
                }
            }
            return true;
        }
    }

    
}
