﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using WindowsFormsApplication1.Events;

namespace WindowsFormsApplication1.SNetwork
{
    public class EquivalenceSearch
    {

        internal static void computeSPEquivalences(List<Configuration> Configurations, List<Configuration> AdditionalConfigs, List<Event> AllEvents)
        {
            if (Configurations.Count < 2)
                return;

            Configuration First, AfetrFirst, BeforeSecound, Secound, Middle;
            List<Event> SeparEvents;
            for (int i = 0; i < Configurations.Count; i++)
            {
                First = Configurations.ElementAt(i);
                AfetrFirst = AdditionalConfigs.ElementAt((i * 3) );
                Middle = AdditionalConfigs.ElementAt((i * 3) + 1);
                BeforeSecound = AdditionalConfigs.ElementAt((i * 3) + 2);
                Secound = Configurations.ElementAt((i + 1) % Configurations.Count);
                
                SeparEvents = Event.getSepartionsBetweenAngles(First.angle, Secound.angle, AllEvents);
                computeSPEquivalences(First, Secound, SeparEvents);

                SeparEvents = Event.getSepartionsBetweenAngles(First.angle, Middle.angle, AllEvents);
                computeSPEquivalences(First, Middle, SeparEvents);

                SeparEvents = Event.getSepartionsBetweenAngles(Middle.angle,Secound.angle, AllEvents);
                computeSPEquivalences(Middle, Secound, SeparEvents);

                SeparEvents = Event.getSepartionsBetweenAngles(First.angle, AfetrFirst.angle, AllEvents);
                computeSPEquivalences(First, AfetrFirst, SeparEvents);

                SeparEvents = Event.getSepartionsBetweenAngles(AfetrFirst.angle, Secound.angle, AllEvents);

                SeparEvents = Event.getSepartionsBetweenAngles(First.angle, BeforeSecound.angle, AllEvents);

                SeparEvents = Event.getSepartionsBetweenAngles(BeforeSecound.angle, Secound.angle, AllEvents);
                computeSPEquivalences(BeforeSecound, Secound, SeparEvents);
            }
        }

        /// <summary>
        /// Pocita ekvivalencie medzi dvomi konfiguraciami potrebujeme aj eventy
        /// </summary>
        /// <param name="conf1"></param>
        /// <param name="conf2"></param>
        internal static void computeSPEquivalences(Configuration conf1, Configuration conf2, List<Event> SeparEvents)
        {
            List<SPoint> EqSPs;
            //pre kazdy SP z prvej konfiguracie
            foreach (SPoint sp in conf1.SPs)
            {
                //najdeme ekvivalentne body z druhej konfiguracie
                EqSPs = sp.ComputeEqSPointsNext(conf2.SPs, SeparEvents);
                //pridame ich do ekvivalentych SP 
                sp.addEqSPs(EqSPs);
                //prejdeme ekvivalentne body k sp a pridame im ho k ekv. bodom
                foreach (SPoint EqSP in EqSPs)
                {
                    EqSP.addEqSP(sp);
                }
            }
        }

        internal static bool areTwoWithoutSepar(List<Sensor> list, List<Event> Separations)
        {
            bool existsEvent;
            //prejdeme cez vsetky dvojice s1 a s2 (s1!=s2) a pozreme, ci sa medzi uhlami nerozisli
            foreach (Sensor s1 in list)
            {
                foreach (Sensor s2 in list)
                {
                    existsEvent = false;
                    if (!s1.Equals(s2))
                    {
                        if (Separations != null)
                            foreach (Event ev in Separations)
                            {
                                //ak sa rozisli, tak tuto dvojicu budeme ignorovat
                                if (ev.ActersPairs.Contains(new PairOfActers(s1, s2)))
                                {
                                    existsEvent = true;
                                    break;
                                }
                            }
                        //ak sa nerozisli, tak vraciame true
                        if (!existsEvent)
                        {
                            return true;
                        }
                    }
                }
            }
            return false;
        }


    }
}
