﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using WindowsFormsApplication1.Events;
using WindowsFormsApplication1.MyMath;

namespace WindowsFormsApplication1.SNetwork
{
    class IntesectionsSearch
    {

        public static void sortIntersections(List<Sensor> Sensors)//vypocitame vzdialenosti bodov a potom ich utriedime podla tychto vzdialenosti
        {
            MyPointWithDist mpwd;
            List<Overlap> ActualOverlaps;
            int maxCount;

            foreach (Sensor s in Sensors)
            {
                ActualOverlaps = s.getActualOverlaps();
                maxCount = s.IntersectionsDynamic.Count + s.IntersectionsStatic.Count + 2 * ActualOverlaps.Count;
                s.ActualSPsWithDist = new List<MyPointWithDist>(maxCount);
                
                //pridame dynamicke priesecniky
                foreach (Intersection inters in s.IntersectionsDynamic)
                {
                    double dist = Distances.distance(s.b, inters.mp);
                    mpwd = new MyPointWithDist(inters.mp, dist);
                    mpwd.Acters = inters.Acters.ToList();
                    s.addMyPointWithDist(mpwd);
                }

                //pridame staticke priesecniky
                foreach (Intersection inters in s.IntersectionsStatic)
                {
                    double dist = Distances.distance(s.b, inters.mp);
                    mpwd = new MyPointWithDist(inters.mp, dist);
                    mpwd.Acters = inters.Acters.ToList();
                    s.addMyPointWithDist(mpwd);
                }

                //spracujeme oba konce prekrytia, ktore ma senzor v aktualnom uhle otocenia
                foreach (Overlap ovl in ActualOverlaps)
                {
                    MyPoint mpB = ovl.ls.b;
                    MyPoint mpE = ovl.ls.e;

                    double distB = Distances.distance(s.b, mpB);
                    double distE = Distances.distance(s.b, mpE);

                    MyPointWithDist pointWithDistB = new MyPointWithDist(mpB, distB);
                    MyPointWithDist pointWithDistE = new MyPointWithDist(mpE, distE);

                    pointWithDistB.Acters = ovl.Acters.ToList();
                    pointWithDistE.Acters = ovl.Acters.ToList();

                    s.addMyPointWithDist(pointWithDistB);
                    s.addMyPointWithDist(pointWithDistE);
                }
                s.ActualSPsWithDist.Sort();
            }
        }

        /// <summary>
        /// hlada prieniky, ktore sa menia pri otacani siete
        /// </summary>
        /// <param name="sn">siet senzorov s ktorou pracuje a vklada jej priamo do listu prienikov najdene prieniky</param>
        public static void searchDynamicIntersectionsOld(SNetwork sn)
        {
            //List<Sensor> closeSensors;
            sn.IntersectionsDynamic.Clear(); //vymazeme vs. dynamicke prieniky siete
            foreach (Sensor s in sn.Sensors)
            {
                s.IntersectionsDynamic.Clear();
            }
            foreach (Sensor s1 in sn.Sensors)
            {
                if (s1 is RSensor) //berieme do uvahy len prieniky rot. senzora so senzormi oboch typov
                {

                    foreach (Sensor s2 in sn.Sensors)
                    {
                        if (!s1.Equals(s2) && !s1.hasOverlapWith(s2)) // ak su to rozne senzory
                        {
                            MyPoint PointOfInters = IntersectionMath.intersectionBeamSensors(s1, s2); //najde prienik, ak sa da
                            if (PointOfInters != null) // ak existuje
                            {
                                Intersection inters = new Intersection(PointOfInters.x, PointOfInters.y);
                                inters.addActers(s1, s2);
                                inters.addActersPair(new PairOfActers(s1, s2));
                                s1.addIntersectionDynamic(inters);
                                s2.addIntersectionDynamic(inters);
                                //ak este nebol najdeny predtym a spracovany - nie je medzi dyn. prienikmi celej siete
                                sn.addIntersectionDynamic(inters);                                
                            }
                        }
                    }
                }
            }
        }


        /// <summary>
        /// hlada prieniky, ktore sa menia pri otacani siete
        /// </summary>
        /// <param name="sn">siet senzorov s ktorou pracuje a vklada jej priamo do listu prienikov najdene prieniky</param>
        public static void searchDynamicIntersections(SNetwork sn)
        {
            List<Sensor> closeSensors;
            List<Sensor>[,] sensorsInMash = Program.cloud.sn.SensorsInMash;

            if (sensorsInMash == null)
            {
                searchDynamicIntersectionsOld(sn);
                return;
            }

            sn.IntersectionsDynamic.Clear(); //vymazeme vs. dynamicke prieniky siete
            foreach (Sensor s in sn.Sensors)
            {
                s.IntersectionsDynamic.Clear();
            }
            for (int i = 0; i < sensorsInMash.GetLength(0); i++)
            {
                for (int j = 0; j < sensorsInMash.GetLength(1); j++)
                {
                    foreach (Sensor s1 in sensorsInMash[i, j])
                    {
                        if (s1 is RSensor) //berieme do uvahy len prieniky rot. senzora so senzormi oboch typov
                        {
                            closeSensors = Program.cloud.sn.getCloseSensors(i, j);

                            foreach (Sensor s2 in closeSensors)
                            {
                                if (!s1.Equals(s2) && !s1.hasOverlapWith(s2)) // ak su to rozne senzory
                                {
                                    MyPoint PointOfInters = IntersectionMath.intersectionBeamSensors(s1, s2); //najde prienik, ak sa da
                                    if (PointOfInters != null) // ak existuje
                                    {
                                        Intersection inters = new Intersection(PointOfInters.x, PointOfInters.y);
                                        inters.addActers(s1, s2);
                                        inters.addActersPair(new PairOfActers(s1, s2));
                                        s1.addIntersectionDynamic(inters);
                                        s2.addIntersectionDynamic(inters);
                                        //ak este nebol najdeny predtym a spracovany - nie je medzi dyn. prienikmi celej siete
                                        sn.addIntersectionDynamic(inters);
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
        /**/
        /// <summary>
        /// najde prieniky lucovych senzorov. Prieniky vklada do senzorov siete sn a do jej zoznamu prienikov (statickych)
        /// </summary>
        /// <param name="sn">siet senzorov s ktorou pracuje a vklada jej priamo do listu prienikov najdene prieniky</param>
        public static void searchStaticIntersections(SNetwork sn)
        {
            List<Sensor> closeSensors;
            List<Sensor>[,] sensorsInMash = Program.cloud.sn.SensorsInMash;

            if (sensorsInMash == null)
            {
                searchDynamicIntersectionsOld(sn);
                return;
            }
            
            sn.IntersectionsStatic.Clear(); //vymazeme vs. staticke prieniky siete

            foreach (Sensor s in sn.Sensors)
            {
                s.IntersectionsStatic.Clear();
            }

            for (int i = 0; i < sensorsInMash.GetLength(0); i++)
            {
                for (int j = 0; j < sensorsInMash.GetLength(1); j++)
                {
                    foreach (Sensor s1 in sensorsInMash[i, j])
                    {
                        if (s1 is BSensor) //berieme do uvahy len prieniky rot. senzora so senzormi oboch typov
                        {
                            closeSensors = Program.cloud.sn.getCloseSensors(i, j);

                            foreach (Sensor s2 in closeSensors)
                            {
                                if (s2 is BSensor) //berieme do uvahy len luc. senzory
                                {
                                    if (!s1.Equals(s2) && !s1.hasOverlapWith(s2)) // ak su to rozne senzory
                                    {
                                        MyPoint PointOfInters = IntersectionMath.intersectionBeamSensors(s1, s2); //najde prienik, ak sa da
                                        if (PointOfInters != null) // ak existuje
                                        {
                                            Intersection inters = new Intersection(PointOfInters.x, PointOfInters.y);
                                            inters.addActers(s1, s2);
                                            inters.addActersPair(new PairOfActers(s1, s2));
                                            s1.addIntersectionStatic(inters);
                                            s2.addIntersectionStatic(inters);
                                            //ak este nebol najdeny predtym a spracovany - nie je medzi dyn. prienikmi celej siete
                                            sn.addIntersectionStatic(inters);
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        /// <summary>
        /// najde prieniky lucovych senzorov. Prieniky vklada do senzorov siete sn a do jej zoznamu prienikov (statickych)
        /// </summary>
        /// <param name="sn">siet senzorov s ktorou pracuje a vklada jej priamo do listu prienikov najdene prieniky</param>
        public static void searchStaticIntersectionsOld(SNetwork sn)
        {
            sn.IntersectionsStatic.Clear(); //vymazeme vs. staticke prieniky siete
            foreach (Sensor s in sn.Sensors)
            {
                s.IntersectionsStatic.Clear();
            }
            foreach (Sensor s1 in sn.Sensors)
            {
                if (s1 is BSensor) //berieme do uvahy len luc. senzory
                {
                    foreach (Sensor s2 in sn.Sensors)
                    {
                        if (s2 is BSensor) //berieme do uvahy len luc. senzory
                        {
                            if (!s1.Equals(s2) && !s1.hasOverlapWith(s2)) // ak su to rozne senzory
                            {
                                MyPoint PointOfInters = IntersectionMath.intersectionBeamSensors(s1, s2); //najde prienik, ak sa da
                                if (PointOfInters != null) // ak existuje
                                {
                                    Intersection inters = new Intersection(PointOfInters.x, PointOfInters.y);
                                    inters.addActers(s1, s2);
                                    inters.addActersPair(new PairOfActers(s1, s2));
                                    s1.addIntersectionStatic(inters);
                                    s2.addIntersectionStatic(inters);
                                    //ak este nebol najdeny predtym a spracovany - nie je medzi dyn. prienikmi celej siete
                                    sn.addIntersectionStatic(inters);
                                }
                            }
                        }
                    }
                }
            }
        }
    }
}
