﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using WindowsFormsApplication1.Graph;
using WindowsFormsApplication1.MyMath;
using WindowsFormsApplication1.Events;

namespace WindowsFormsApplication1.SNetwork
{
    class SAreasSearch
    {
        static List<List<int>> cycles;
        public static int areaIndex = 0;

        public static List<SimpleArea> searchAreas(Graph<SPoint> IndGraph)
        {

            SPoint SPHelp;
            List<SPoint> listSP;
            SortEdgesClockWise(IndGraph);
            cycles = new List<List<int>>(20);
            foreach (Vertex<SPoint> v in IndGraph.Vertices)
            {
                findNeighbouringFaces(v, IndGraph);
            }

            List<SimpleArea> areas = new List<SimpleArea>(20);
            SimpleArea area;

            foreach (List<int> cycle in cycles)
            {
                
                listSP = new List<SPoint>(20);
                foreach (int index in cycle)
                {           
                    SPHelp = IndGraph.GetByIndex(index).Data;
                    listSP.Add(SPHelp);
                }

                area = new SimpleArea(listSP);
                area.actualIndex = areaIndex;
                area.index = areaIndex;
                areaIndex++;
                areas.Add(area);
            }
            return areas;
        }

        public static void SortEdgesClockWise(Graph<SPoint> G)
        {
            foreach (Vertex<SPoint> v1 in G.Vertices)
            {
                sortEdgesOfVertex(G, v1);
            }
        }

        private static void sortEdgesOfVertex(Graph<SPoint> g, Vertex<SPoint> v1)
        {
            List<Vertex<SPoint>> neighbours = g.ListOfIndexesToListOfVertices(v1.Edges);
            MyPoint p = v1.Data.Location;
            MyPoint upper = new MyPoint(p.x, p.y - 1);

            foreach (Vertex<SPoint> v in neighbours)
            {
                double angle = Angles.angleTriangle(upper, p, v.Data.Location);
                if (p.x > v.Data.Location.x)
                {
                    angle = 360 - angle;
                }
                EdgeWithAngle aoe = new EdgeWithAngle(v1.Index, v.Index, angle);
                if (!v1.EdgesWithAngles.Contains(aoe))
                    v1.EdgesWithAngles.Add(aoe);
            }
            v1.EdgesWithAngles.Sort();

        }

        private  static void findNeighbouringFaces(Vertex<SPoint> v, Graph<SPoint> G)
        {
             List<int> cycle;
            Vertex<SPoint> PrevV;
            foreach (EdgeWithAngle e0 in v.EdgesWithAngles)
            {
                if (!e0.flagVisited)
                {
                    cycle = new List<int>(20);
                    e0.flagVisited = true;
                    cycle.Add(e0.indexBVertex);
                    EdgeWithAngle eNext = G.GetNextEdgeWithAngle(e0);
                    PrevV = v;
  
                    while (!e0.Equals(eNext))
                    {
                        eNext.flagVisited = true;
                        cycle.Add(eNext.indexBVertex);
                        eNext = G.GetNextEdgeWithAngle(eNext);
                    }
                    cycles.Add(cycle);
                }
            }
        }

        

        public static List<SimpleArea> getEmptyAreas(Configuration conf)
        {

            List<SimpleArea> result = new List<SimpleArea> ();
            if (conf.Overlaps != null)
            {
               if (conf.Overlaps.Count >0){
                   result =  makeEmptyAreasFromOverlaps(conf);
                  
               }
                foreach (SimpleArea sa in searchIntersecionsOfThree(conf)){
                    result.Add(sa);
                }
                
            }
            return result;
        }

        private static List<SimpleArea> makeEmptyAreasFromOverlaps(Configuration conf)
        {
            List<SimpleArea> result = new List<SimpleArea>(conf.Overlaps.Count);
         
            List<SPoint>  V;
            foreach (Overlap ovl in conf.Overlaps){
                V = new List<SPoint> (2);
                V.Add(conf.GetSPWithLoc(ovl.ls.b));
                V.Add(conf.GetSPWithLoc(ovl.ls.e));

                result.Add(new SimpleArea(V, conf.angle));
            }
            return result;
        }

        private static List<SimpleArea> searchIntersecionsOfThree(Configuration conf)
        {

            List<SimpleArea> result = new List<SimpleArea>(4);

            List<SPoint> V;
            //pridame SP, ktore maju aspon 3 vlasnikov
            foreach (SPoint sp in conf.SPs)
            {
                if (sp.Owners.Count > 2)
                {
                    V = new List<SPoint> { sp };
                    result.Add(new SimpleArea(V, conf.angle));
                }
            }

            foreach (Event ev in conf.Events)
            {
                if (ev.type == Event.UNDEFINED)
                {
                    SimpleArea sa = makeSAreasFromUndef(ev, conf);
                    if (sa != null)
                        result.Add(sa);
                }
            }

            return result;

        }

        /// <summary>
        /// z eventu stretu troch spravime oblast prazdnu...
        /// </summary>
        /// <param name="ev"></param>
        /// <param name="conf"></param>
        /// <returns></returns>
        private static SimpleArea makeSAreasFromUndef(Event ev,Configuration conf)
        {

            List<SPoint> V,V1,V2,V3;
            
            Sensor s1, s2, s3;
            if (ev.Acters.Count != 3)
            {
                throw new Exception("Akteri maju byt prave traja!");
            }
            //aktery udalost stretnutia troch
            s1 = ev.Acters.ElementAt(0);
            s2 = ev.Acters.ElementAt(1);
            s3 = ev.Acters.ElementAt(2);
            //spolocne SP body tychto senzorov
            V1 = conf.getCommonSP(s1, s2);
            V2 = conf.getCommonSP(s2, s3);
            V3 = conf.getCommonSP(s1, s3);
            //ak maju prave jeden spolocny bod dvojice tychto troch senzorov
            //lebo ked maju viacero spol bodov, tak ide o prekrytie a to nas nezaujima
            if (V1.Count == 1 && V2.Count == 1 && V3.Count == 1)
            {
                //vytvorime oblast, ktora bude prazdnou oblastou
                V = new List<SPoint>(3);
                V.Add(V1.ElementAt(0));
                if (!V.Contains(V2.ElementAt(0)))
                {
                    V.Add(V2.ElementAt(0));
                }
                if (!V.Contains(V3.ElementAt(0)))
                {
                    V.Add(V3.ElementAt(0));
                }
                return new SimpleArea(V, conf.angle);
            }
            return null;
        }

        private static SimpleArea getAreaWithSP(List<SimpleArea> result, SPoint sPoint)
        {
            foreach (SimpleArea sa in result)
            {
                foreach (SPoint sp in sa.V)
                {
                    if (sPoint.Equals(sp))
                        return sa;
                }
            }
            return null;
        }


        internal static void searchSpecialAreas(List<Configuration> AdditionalConfigs)
        {


            //najdeme oblati pre kazdu konf.
            //a z kazdej pridavnej configuracie, ktora ma specialny graf, odstranime oblasti, ktore su vramci jedneho rozmnozenia bodu
            foreach (Configuration conf in AdditionalConfigs)
            {
                if (conf.SpecialGraph != null)
                {
                    //zoznam, ktore budeme vymazavat
                    List<SimpleArea> InCompleteGraph = new List<SimpleArea>();

                    //najdeme vs. oblasti specialneho grafu tejto konfiguracie
                    conf.SpecialSAreas = SAreasSearch.searchAreas(conf.SpecialGraph);

                    conf.SpecialParts = ConfigurationParts.makeParts(conf.SpecialSAreas, conf.SpecialGraph);

                    //prejdeme vs. oblasti a zistime, ktore treba vymazat
                    foreach (SimpleArea sa in conf.SpecialSAreas)
                    {
                        if (ItIsInCompletedGraph(sa, conf.SpecialGraph))
                            InCompleteGraph.Add(sa);
                        sa.Angle = conf.angle;
                    }
                    foreach (SimpleArea sa in InCompleteGraph)
                        conf.SpecialSAreas.Remove(sa);
                } 
            }


            foreach (Configuration conf in AdditionalConfigs)
            {
                if (conf.SpecialGraph != null)
                {
                    foreach (SimpleArea sa in conf.SpecialSAreas)
                    {
                        sa.computeIndexesOfSpecEgdes(conf.SpecialGraph);
                        sa.neighbours = new List<int>();
                    }

                    foreach (SimpleArea sa1 in conf.SpecialSAreas)
                    {
                        foreach (SimpleArea sa2 in conf.SpecialSAreas)
                        {
                            if (!sa1.Equals(sa2))
                            {
                                if (SimpleArea.hasCommonExtraEdge(sa1, sa2))
                                {
                                    
                                    if (!sa2.neighbours.Contains(sa1.index))
                                    {
                                        sa2.neighbours.Add(sa1.index);
                                    }

                                    if (!sa1.neighbours.Contains(sa2.index))
                                    {
                                        sa1.neighbours.Add(sa2.index);
                                    }

                                    foreach (int neigh in sa1.neighbours)
                                    {
                                        if (!sa2.neighbours.Contains(neigh))
                                        {
                                            sa2.neighbours.Add(neigh);
                                        }
                                    }

                                    foreach (int neigh in sa2.neighbours)
                                    {
                                        if (!sa1.neighbours.Contains(neigh))
                                        {
                                            sa1.neighbours.Add(neigh);
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }

        }

        private static bool ItIsInCompletedGraph(SimpleArea sa, Graph<SPoint> SpecialGraph)
        {
            

            SPoint sp1, sp2;
            int index;
            SPoint commonEq=null;
            //zoznam ktore indexy sa vyskytuju na hranici jedn. oblasti
        
            List<SPoint> EqSPsWithFirst = sa.V.First().getEqSPsWithAngle(SpecialGraph.angleOfConf);

            bool hasCommonEqPoint = false;
            bool isItActual = false;
            foreach (SPoint EqSP in EqSPsWithFirst)
            {
                isItActual = true;
                foreach (SPoint sp in sa.V)
                {
                    //pri specialnych robime vynimku - tie su vzdy vramci kompl grafoch
                    if (!sp.hasEqSP(EqSP) && !sp.speacial && !sp.AddedAsEventLocation)
                    {
                        isItActual =false;
                        break;
                    }
                }
                if (isItActual)
                {
                    hasCommonEqPoint = true;
                    commonEq = EqSP;
                    break;
                }
            }
            //ak nemaju spolocny eq SP tak nemozu tvorit vnutro jedneho kompl grafu.
            if (!hasCommonEqPoint)
                return false;

            List<int> indexesOfExtraEdges = new List<int>();
            //prejdeme vsetky hrany
            for (int i = 0; i < sa.V.Count; i++)
            {
                sp1 = sa.V.ElementAt(i);
                sp2 = sa.V.ElementAt((i+1) % sa.V.Count);
                index = SpecialGraph.getIndexOfExtraEdge(sp1, sp2);
                //staci jedna hrana s takym indexom, ak nie - toto zakomentovat a odkom. podtym
                if (index == commonEq.GraphIndex)
                    return true;
                
            }
            //neobsahuje dve spec hrany s rovnakym indexom
            return false;
        }
    }
}
